-- GLOBALS: unpack, select, LibStub, ITEM_SEARCHBAR_LIST
-- GLOBALS: VoidStorageFrame, ReagentBankFrame, MerchantFrame

--[[//////////////////////////////////////////////////

    GLO-LO

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    local _G = _G
    local math = math
    local tostring = tostring
    local tonumber = tonumber
    local string = string
    local pairs = pairs
    local format = format
    local tinsert = tinsert
    local strmatch = strmatch
    local strsplit = strsplit

    local CreateFrame = CreateFrame
    local UIParent = UIParent
    local ResetCursor = ResetCursor
    local SetCursor = SetCursor
    local ShowInspectCursor = ShowInspectCursor
    local GameTooltip = GameTooltip
    local GameTooltip_Hide = GameTooltip_Hide
    local GameTooltip_ShowCompareItem = GameTooltip_ShowCompareItem
    local BattlePetToolTip_Show = BattlePetToolTip_Show
    local BattlePetTooltip = BattlePetTooltip
    local CloseDropDownMenus = CloseDropDownMenus
    local ToggleDropDownMenu = ToggleDropDownMenu
    local UIDropDownMenu_CreateInfo = UIDropDownMenu_CreateInfo
    local UIDropDownMenu_SetWidth = UIDropDownMenu_SetWidth
    local UIDropDownMenu_Initialize = UIDropDownMenu_Initialize
    local UIDropDownMenu_AddButton = UIDropDownMenu_AddButton
    local UseContainerItem = UseContainerItem
    local PickupContainerItem = PickupContainerItem
    local PickupGuildBankItem = PickupGuildBankItem
    local GetItemQualityColor = GetItemQualityColor
    local SplitContainerItem = SplitContainerItem
    local SplitGuildBankItem = SplitGuildBankItem
    local ClickVoidStorageSlot = ClickVoidStorageSlot
    local GetCurrentGuildBankTab = GetCurrentGuildBankTab
    local AutoStoreGuildBankItem = AutoStoreGuildBankItem
    local GetCVarBool = GetCVarBool
    local IsShiftKeyDown = IsShiftKeyDown
    local IsModifiedClick = IsModifiedClick
    local IsControlKeyDown = IsControlKeyDown
    local HandleModifiedItemClick = HandleModifiedItemClick
    local SocketContainerItem = SocketContainerItem
    local GetMoneyString = GetMoneyString
    local ColorPickerFrame = ColorPickerFrame
    local FauxScrollFrame_OnVerticalScroll = FauxScrollFrame_OnVerticalScroll
    local IsBagOpen, OpenBag = IsBagOpen, OpenBag
    local IsArtifactPowerItem = IsArtifactPowerItem
    local SocketInventoryItem = SocketInventoryItem
    local SpellCanTargetItem = SpellCanTargetItem
    local SpellCanTargetItemID = SpellCanTargetItemID
    local DressUpItemLink = DressUpItemLink
    local SplitStack = SplitStack
    local ContainerFrameItemButton_OnClick = ContainerFrameItemButton_OnClick
    local StackSplitFrame = StackSplitFrame

    local B, L, T = unpack(select(2, ...))
    B.Bank = {['Tooltip'] = {}, ['Bags'] = {}, ['Default'] = {}, ['Bag'] = {}}


--[[//////////////////////////////////////////////////

    CREATE FRAME AND BUTTONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function B.Bank:CreateFrame()

        local frame = CreateFrame("Frame", "BWbank", T.F.mainframe, "BackdropTemplate")
        frame:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
        frame:SetFrameStrata("HIGH")
        frame:SetScript("OnShow", B.Bank.Show)
        frame:SetScript("OnHide", B.Bank.Hide)

        B.I.Backdrop(frame)
        T.F.mainframe.bank = frame
    end

    function B.Bank:Show()
        T.F.mainframe.bankbutton:LockHighlight()
    end

    function B.Bank:Hide()
        T.F.mainframe.bankbutton:UnlockHighlight()
    end


    function B.Bank:CreateButton()

        local button = B.I.Main.Frame.Button("|cffffffff"..L[10001].."|r")
        button:SetBackdropColor(T.BANKTAB.r, T.BANKTAB.g, T.BANKTAB.b, T.BANKTAB.a)
        button:SetScript("OnClick", B.Bank.Click)

        T.F.mainframe.bankbutton = button
    end

    function B.Bank:Click(...)

        local frame = self:GetParent()

        if not T.ISBANKOPEN then
            frame.character:Hide()
        elseif T.ISBANKOPEN and frame.character:IsVisible() then
            -- move bags
            B.AnchorBags(frame.bank)
        end

        frame.reagent:Hide()
        frame.void:Hide()
        frame.guild:Hide()
        frame.search:Hide()

        if frame.bank:IsVisible() then
            frame.lastbank = nil
            frame.bank:Hide()
        else
            frame.lastbank = "bank"
            frame.bank:Show()
        end
    end



    function B.Bank:CreateLayout()

        T.F.mainframe.bank.tooltiptoggle = B.I.Create_FrameButton(L[10058], T.F.mainframe.bank, B.Bank.Tooltip)
        T.F.mainframe.bank.bagstoggle = B.I.Create_FrameButton(L[10057], T.F.mainframe.bank, B.Bank.Bags)

        B.Bank:CreateBags()
        B.Bank:CreateContainers()
        B.Bank:CreateButtons()
    end


    function B.Bank.Tooltip:Click(...)
        T.CONFIG['bankbagtooltiptoggle'] = not T.CONFIG['bankbagtooltiptoggle']
    end

    function B.Bank.Tooltip:Enter()

        GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
        GameTooltip:SetText(L[10009])
        GameTooltip:Show()
    end

    function B.Bank.Tooltip:Leave()
        GameTooltip:Hide()
    end


    function B.Bank.Bags:Click(...)

        if T.F.mainframe.bank.bags:IsVisible() then
            T.F.mainframe.bank.bags:Hide()
            T.CONFIG['bankbagstoggle'] = false
        else
            T.F.mainframe.bank.bags:Show()
            T.CONFIG['bankbagstoggle'] = true
        end
    end

    function B.Bank.Bags:Enter()

        GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
        GameTooltip:SetText(L[10008])
        GameTooltip:Show()
    end

    function B.Bank.Bags:Leave()
        GameTooltip:Hide()
    end


    function B.Bank:CreateBags()

        local x
        T.F.mainframe.bank.bags = CreateFrame("Frame", nil, T.F.mainframe.bank, "BackdropTemplate")

        for x = 1, T.NUM_BANKBAGSLOTS + 1 do
            T.F.mainframe.bank.bags[x] = {}
            T.F.mainframe.bank.bags[x].tab = B.I.Tab.Create(T.F.mainframe.bank.bags, "BWbank"..x, "BackdropTemplate")
            T.F.mainframe.bank.bags[x].tab.search:SetAllPoints(T.F.mainframe.bank.bags[x].tab)
            T.F.mainframe.bank.bags[x].tab.index = x + 3
            T.F.mainframe.bank.bags[x].tab:SetScript("OnEnter", self.Bag.Enter)
            T.F.mainframe.bank.bags[x].tab:SetScript("OnLeave", self.Bag.Leave)

            if x == 1 then --Backpack
                T.F.mainframe.bank.bags[x].tab:SetNormalTexture("Interface\\Buttons\\Button-Backpack-Up")
                T.F.mainframe.bank.bags[x].tab.index = x
            end
        end
    end

    function B.Bank.Bag:Click()
        if not T.F.mainframe.alt and T.ISBANKOPEN then ToggleBag(self.index) end
    end

    function B.Bank.Bag:Enter()

        if (T.CONFIG and T.CONFIG['bankbagtooltiptoggle']) or not T.CONFIG then
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT", -10, -10)
            if self.link then GameTooltip:SetHyperlink(self.link) end
            if self.quality then
                local r, g, b = GetItemQualityColor(self.quality)
                GameTooltip:SetBackdropBorderColor(r, g, b)
            end
            GameTooltip:Show()
        end

        if #T.BW.items == 0 then
            B.Bank.Bag:Search(self.index)
        end
    end

    function B.Bank.Bag:Leave()

        GameTooltip:Hide()

        if #T.BW.items == 0 then
            B.Bank.Bag:Search(false)
        end
    end


    function B.Bank:CreateContainers()
        -- Create parent frames for buttons to communicate with blizzard api for clicking behavior.
        local x
        T.F.mainframe.bank.container = CreateFrame("Frame", nil, T.F.mainframe.bank)
        T.F.mainframe.bank.container[T.BANK_CONTAINER] = {}
        B.I.Frame.Create(T.F.mainframe.bank, T.F.mainframe.bank.container[T.BANK_CONTAINER])
        T.F.mainframe.bank.container[T.BANK_CONTAINER].frame:SetID(T.BANK_CONTAINER)

        for x = T.ITEM_INVENTORY_BANK_BAG_OFFSET + 1, T.ITEM_INVENTORY_BANK_BAG_OFFSET + T.NUM_BANKBAGSLOTS do
            T.F.mainframe.bank.container[x] = {}
            B.I.Frame.Create(T.F.mainframe.bank, T.F.mainframe.bank.container[x])
            T.F.mainframe.bank.container[x].frame:SetID(x)
        end
    end

    function B.Bank:CreateButtons()
        -- Create all buttons using the max slot size for bags. Hide the extra uneeded slots.
        local x, b, button, rows

        B.I.Frame.Create(T.F.mainframe.bank, T.F.mainframe.bank)

        for b = 1, T.NUM_BANKGENERIC_SLOTS do
            button = B.I.Button.Create(b, T.F.mainframe.bank.container[T.BANK_CONTAINER].frame)
            button:SetScript("OnEnter", B.I.Button.Enter)
            button:SetScript("OnLeave", B.I.Button.Leave)
            button.SplitStack = SplitStack
            T.F.mainframe.bank.container[T.BANK_CONTAINER].buttons[#T.F.mainframe.bank.container[T.BANK_CONTAINER].buttons + 1] = button
        end

        for x = T.ITEM_INVENTORY_BANK_BAG_OFFSET + 1, T.ITEM_INVENTORY_BANK_BAG_OFFSET + T.NUM_BANKBAGSLOTS do
            for b = 1, T.MAX_BAG_SLOTS do
                button = B.I.Button.Create(b, T.F.mainframe.bank.container[x].frame)
                button:Hide()
                button:SetScript("OnEnter", B.I.Button.Enter)
                button:SetScript("OnLeave", B.I.Button.Leave)
                button.SplitStack = SplitStack
                T.F.mainframe.bank.container[x].buttons[#T.F.mainframe.bank.container[x].buttons + 1] = button
            end
        end
    end

    function B.Bank:UpdateButtons()
        --Load the number of slots dynamically, based on the loaded character or alt database.
        local x, b, button, rows
        local bank = B.GetPlayerBank('bank')

        for b = T.NUM_BANKGENERIC_SLOTS + 1, #T.F.mainframe.bank.buttons do
            T.F.mainframe.bank.buttons[b]:Hide()
        end
        T.F.mainframe.bank.buttons = {}

        for b = 1, T.NUM_BANKGENERIC_SLOTS do
            T.F.mainframe.bank.buttons[#T.F.mainframe.bank.buttons + 1] = T.F.mainframe.bank.container[T.BANK_CONTAINER].buttons[b]
        end

        for x = T.ITEM_INVENTORY_BANK_BAG_OFFSET + 1, T.ITEM_INVENTORY_BANK_BAG_OFFSET + T.NUM_BANKBAGSLOTS do
            if bank[x] then
                for b = 1, bank[x]['slots'] do
                    button = T.F.mainframe.bank.container[x].buttons[b]
                    button:Show()
                    T.F.mainframe.bank.buttons[#T.F.mainframe.bank.buttons + 1] = button
                end
            end
        end

        rows = B.I.Grid.Update(T.F.mainframe.bank)
        T.F.mainframe.bank:SetHeight(10 + (T.BUTTON_HEIGHT * (rows + 1)) + (T.FRAMEWHITESPACE * rows))
        T.F.mainframe.bank:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
    end


--[[//////////////////////////////////////////////////

    FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function B.Bank:IsGenericSlot(button)

        local name = button:GetName()
        if name and strfind(name, "BankFrameItem%d+") then
            return true
        end
        return false
    end

    function B.Bank:IsBag(button)

        local name = button:GetName()
        if name and strfind(name, "ContainerFrame%d+Item%d+") then
            local bagid = button:GetParent():GetID()
            if bagid >= (T.NUM_BAG_SLOTS + 1) and bagid <= (T.NUM_BAG_SLOTS + T.NUM_BANKBAGSLOTS) then
                return true
            end
        end
        return false
    end


    function B.Bank:Update()

        local page, total, counter, quality, r, g, b
        local bank = B.GetPlayerBank('bank')

        B.Bank:UpdateButtons()

        counter = 0
        for page = 1, T.NUM_BAG_SLOTS + T.NUM_BANKBAGSLOTS do
            if bank[page] then

                if page > 1 then -- Ignore generic bank
                    B.UpdateBagTab(T.F.mainframe.bank.bags[page - 3].tab, bank[page])
                end
            
                B.UpdatePage(T.F.mainframe.bank, bank[page]['slots'], bank[page], counter)
                counter = counter + bank[page]['slots']
            
            elseif page >= T.NUM_BAG_SLOTS + 1 then -- Update bags display
               B.UpdateBagTab(T.F.mainframe.bank.bags[page - 3].tab, {})
            end
        end
    end

    function B.Bank:Scan()
        -- This is used to force scan the entire bank in the event the user is using a bag addon that
        -- modifies the blizzard frame and makes it impossible to scan.
        if not T.ISBANKOPEN then return end -- Circumvent early BAG_UPDATE
        local x, counter, count, link, bag, slot, slots
        local bank = T['bank']

        if not bank[1] then bank[1] = B.newtable() end
        for x = 1, T.NUM_BANKGENERIC_SLOTS do
            _, count, _, _, _, _, link = GetContainerItemInfo(T.BANK_CONTAINER, x)

            if link then
                bank[1][x] = bank[1][x] or B.newtable()
                bank[1][x]['count'] = count
                bank[1][x]['link'] = link
            else
                bank[1][x] = B.deltable(bank[1][x])
            end
        end

        for bag = T.ITEM_INVENTORY_BANK_BAG_OFFSET + 1, T.ITEM_INVENTORY_BANK_BAG_OFFSET + T.NUM_BANKBAGSLOTS do
            if not bank[bag] then bank[bag] = B.newtable() end
            
            local inventoryID = BankButtonIDToInvSlotID(bag, 1) - 4
            local baglink = GetInventoryItemLink("player", inventoryID)

            slots = GetContainerNumSlots(bag)
            bank[bag]['slots'] = slots
            bank[bag]['link'] = baglink

            for x = 1, slots do
                _, count, _, _, _, _, link = GetContainerItemInfo(bag, x)

                if link then
                    bank[bag][x] = bank[bag][x] or B.newtable()
                    bank[bag][x]['link'] = link
                    bank[bag][x]['count'] = count > 1 and count or nil
                else
                    bank[bag][x] = B.deltable(bank[bag][x])
                end
            end
        end

        if B.IsCurrentPlayer() then self:Update() end
        B.CacheMainItems('bank')
    end

    function B.Bank:ScanBag(bag)

        local x, slots, count, link
        local bank = T['bank']

        B.Bank.Bag:Update(bank, bag)

        for x = 1, bank[bag]['slots'] do
            _, count, _, _, _, _, link = GetContainerItemInfo(bag, x)
            bank[bag][x] = B.Scan(bank[bag][x], link, count)
        end

        if B.IsCurrentPlayer() then self:Update() end
        B.CacheMainItems('bank')
    end

    function B.Bank:ScanGenericSlot(slot)

        local bank = T['bank']
        local _, count, _, _, _, _, link = GetContainerItemInfo(T.BANK_CONTAINER, slot)

        if not B.CompareSlot(bank[1][slot], link, count) then

            bank[1][slot] = B.Scan(bank[1][slot], link, count)
            if B.IsCurrentPlayer() then self:UpdateGenericSlot(slot) end
            B.CacheMainItems('bank')
        end
    end

    function B.Bank:UpdateGenericSlot(slot)

        local bank = B.GetPlayerBank('bank')
        B.UpdateSlot(T.F.mainframe.bank.buttons[slot], bank[1][slot])
    end

    function B.Bank:ScanSlot(bag, slot)

        local bank = T['bank']
        local _, count, _, _, _, _, link = GetContainerItemInfo(bag, slot)

        B.Bank.Bag:Update(bank, bag)

        if not B.CompareSlot(bank[bag][slot], link, count) then

            bank[bag][slot] = B.Scan(bank[bag][slot], link, count)
            if B.IsCurrentPlayer() then self:UpdateSlot(B.GetSlotID(bag, slot), bag, slot) end
            B.CacheMainItems('bank')
        end
    end

    function B.Bank:UpdateSlot(button, page, slot)

        local bank = B.GetPlayerBank('bank')
        B.UpdateSlot(T.F.mainframe.bank.buttons[button], bank[page][slot])
    end

    function B.Bank.Bags:Update()

        local total, page
        local bank = T['bank']

        for bag = T.ITEM_INVENTORY_BANK_BAG_OFFSET + 1, T.ITEM_INVENTORY_BANK_BAG_OFFSET + T.NUM_BANKBAGSLOTS do
            B.Bank.Bag:Update(bank, bag)
            B.UpdateBagTab(T.F.mainframe.bank.bags[bag - 3].tab, bank[bag])
        end

        B.Bank:UpdateButtons()
    end

    function B.Bank.Bag:Update(bank, bag)

        if not bank[bag] then bank[bag] = B.newtable() end

        local inventoryID = BankButtonIDToInvSlotID(bag, 1) - 4
        local baglink = GetInventoryItemLink("player", inventoryID)
        local slots = GetContainerNumSlots(bag)

        bank[bag]['slots'] = slots
        bank[bag]['link'] = baglink

        B.Bank:UpdateButtons()
    end

    function B.Bank:Search(text)

        local x, page, counter, button, buttons, name, tab
        local bank = B.GetPlayerBank('bank')

        local counter = 1
        for page = 1, #bank do
            
            if bank[page] then
                tab = page == 1 and page or page - 3
                
                if text then 
                    T.F.mainframe.bank.bags[tab].tab.search:Show()
                else 
                    T.F.mainframe.bank.bags[tab].tab.search:Hide() 
                end
                buttons = T.F.mainframe.bank.buttons

                for x = 1, bank[page]['slots'] do
                    button = buttons[counter]

                    if button.name and text then
                        if B.find(text, button) then
                            button.loc = "bank."..tab
                            B.Search:Slot(button)
                            T.F.mainframe.bank.bags[tab].tab.search:Hide()
                        else
                            button.search:Show()
                        end
                    else
                        button.search:Hide()
                    end
                    counter = counter + 1
                end
            end
        end
    end

    function B.Bank.Bag:Search(bag)

        local bank = B.GetPlayerBank('bank')
        local x, page, counter, button, buttons

        local counter = 1
        for page = 1, #bank do
            if bank[page] then

                if not bag or page == bag then
                    for x = 1, bank[page]['slots'] do
                        T.F.mainframe.bank.buttons[counter].search:Hide()
                        counter = counter + 1
                    end
                else
                    for x = 1, bank[page]['slots'] do
                        T.F.mainframe.bank.buttons[counter].search:Show()
                        counter = counter + 1
                    end
                end
            end
        end
    end


--[[//////////////////////////////////////////////////

    POSITION

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function B.Bank:Anchor(startpoint, endpoint, from, to)
        local lasttab

        T.F.mainframe.bank:ClearAllPoints()
        T.F.mainframe.bank:SetPoint(startpoint..from, T.F.mainframe, endpoint..from)

        B.I.Main.Frame.ButtonHighlight(T.F.mainframe.bankbutton, endpoint)

        for x = 1, T.NUM_BANKBAGSLOTS + 1 do
            T.F.mainframe.bank.bags[x].tab:ClearAllPoints()
            if x == 1 then
                T.F.mainframe.bank.bags[x].tab:SetPoint("TOP"..from, T.F.mainframe.bank, "TOP"..to)
            else
                T.F.mainframe.bank.bags[x].tab:SetPoint("TOP"..from, lasttab, "BOTTOM"..from, 0, 0)
            end
            lasttab = T.F.mainframe.bank.bags[x].tab
        end

        T.F.mainframe.bank.tooltiptoggle:ClearAllPoints()
        T.F.mainframe.bank.tooltiptoggle:SetPoint("TOP"..to, to == "RIGHT" and -10 or 10, 0)

        T.F.mainframe.bank.bagstoggle:ClearAllPoints()
        T.F.mainframe.bank.bagstoggle:SetPoint("TOP"..to, T.F.mainframe.bank.tooltiptoggle, "TOP"..from, to == "RIGHT" and -5 or 5, 0)
    end
