-- GLOBALS: unpack, select, LibStub, ITEM_SEARCHBAR_LIST
-- GLOBALS: VoidStorageFrame, ReagentBankFrame, MerchantFrame

--[[//////////////////////////////////////////////////

    GLO-LO

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    local _G = _G
    local math = math
    local tostring = tostring
    local tonumber = tonumber
    local string = string
    local pairs = pairs
    local format = format
    local tinsert = tinsert
    local strmatch = strmatch
    local strsplit = strsplit

    local CreateFrame = CreateFrame
    local UIParent = UIParent
    local ResetCursor = ResetCursor
    local SetCursor = SetCursor
    local ShowInspectCursor = ShowInspectCursor
    local GameTooltip = GameTooltip
    local GameTooltip_Hide = GameTooltip_Hide
    local GameTooltip_ShowCompareItem = GameTooltip_ShowCompareItem
    local BattlePetToolTip_Show = BattlePetToolTip_Show
    local BattlePetTooltip = BattlePetTooltip
    local CloseDropDownMenus = CloseDropDownMenus
    local ToggleDropDownMenu = ToggleDropDownMenu
    local UIDropDownMenu_CreateInfo = UIDropDownMenu_CreateInfo
    local UIDropDownMenu_SetWidth = UIDropDownMenu_SetWidth
    local UIDropDownMenu_Initialize = UIDropDownMenu_Initialize
    local UIDropDownMenu_AddButton = UIDropDownMenu_AddButton
    local UseContainerItem = UseContainerItem
    local PickupContainerItem = PickupContainerItem
    local PickupGuildBankItem = PickupGuildBankItem
    local GetItemQualityColor = GetItemQualityColor
    local SplitContainerItem = SplitContainerItem
    local SplitGuildBankItem = SplitGuildBankItem
    local ClickVoidStorageSlot = ClickVoidStorageSlot
    local GetCurrentGuildBankTab = GetCurrentGuildBankTab
    local AutoStoreGuildBankItem = AutoStoreGuildBankItem
    local GetCVarBool = GetCVarBool
    local IsShiftKeyDown = IsShiftKeyDown
    local IsModifiedClick = IsModifiedClick
    local IsControlKeyDown = IsControlKeyDown
    local HandleModifiedItemClick = HandleModifiedItemClick
    local SocketContainerItem = SocketContainerItem
    local GetMoneyString = GetMoneyString
    local ColorPickerFrame = ColorPickerFrame
    local FauxScrollFrame_OnVerticalScroll = FauxScrollFrame_OnVerticalScroll
    local IsBagOpen, OpenBag = IsBagOpen, OpenBag
    local IsArtifactPowerItem = IsArtifactPowerItem
    local SocketInventoryItem = SocketInventoryItem
    local SpellCanTargetItem = SpellCanTargetItem
    local SpellCanTargetItemID = SpellCanTargetItemID
    local DressUpItemLink = DressUpItemLink
    local SplitStack = SplitStack
    local ContainerFrameItemButton_OnClick = ContainerFrameItemButton_OnClick
    local StackSplitFrame = StackSplitFrame
    local Search = LibStub:GetLibrary('LibItemSearch-1.0')

    local B, L, T = unpack(select(2, ...))
    B.Search = {['Count'] = {}}


--[[//////////////////////////////////////////////////

    CREATE FRAME AND BUTTONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function B.Search:CreateFrame()

        local frame = CreateFrame("Frame", "BWsearch", T.F.mainframe, "BackdropTemplate")
        frame:SetHeight(200)
        frame:EnableMouse(true)

        T.F.mainframe.search = frame
    end


    function B.Search:CreateLayout()

        self:CreateEditBox()
        self.Count:CreateButton()
        self:CreateScroll()
    end


    function B.Search:CreateEditBox()

        local editbox = CreateFrame("EditBox", "BW_Searchbar", T.F.mainframe, "BagSearchBoxTemplate")
        editbox:SetHeight(25)
        editbox:SetWidth(192)
        editbox:SetAutoFocus(false)
        editbox:SetPoint("LEFT", T.F.mainframe.guildbutton, "RIGHT", 10, 0)
        editbox:SetHyperlinksEnabled(true)
        editbox:SetMultiLine(false)
        editbox:SetMaxLetters(75)

        local fontName, fontHeight = editbox:GetFont()
        editbox:SetFont(fontName, fontHeight + 2)

        editbox:HookScript("OnEscapePressed", self.EscapePressed)
        editbox:HookScript("OnTextChanged", self.TextChanged)

        B.I.RemoveEdge(editbox)
        tinsert(ITEM_SEARCHBAR_LIST, "BW_Searchbar")
        T.F.mainframe.searchbar = editbox
    end

    function B.Search:TextChanged(userInput)

        local text = self:GetText()
        T.F.mainframe.countbutton.count = 0

        if text == "" or text == L[9007] then
            T.F.mainframe:SetBackdropColor(T.THEME.r, T.THEME.g, T.THEME.b, T.THEME.a)
            T.F.mainframe:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, T.THEME.a)
        end

        if text == L[9007] or text == "" then
            B.Search:Search(false)
        else
            T.F.mainframe:SetBackdropColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
            T.F.mainframe:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
            B.Search:Search(text)
        end
    end

    function B.Search:EscapePressed()
        self:ClearFocus()
    end


    function B.Search.Count:CreateButton()

        local button = B.I.Main.Frame.Button("|cffffffff|r")
        button:SetWidth(64)
        button:SetPoint("LEFT", T.F.mainframe.searchbar, "RIGHT")
        button:SetBackdropColor(T.THEME.r, T.THEME.g, T.THEME.b, T.THEME.a)
        button.count = 0

        button:SetScript("OnHide", self.Hide)
        button:SetScript("OnClick", self.Click)

        button.labelcount = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        button.labelcount:SetFont(T.CONFIG['FontQuantity'] == "default" and T.FONTS[2] or not T.CONFIG['FontQuantity'] and T.FONTS[2] or T.FONTS['quantity'], 14) -- Resize string
        button.labelcount:SetTextColor(0, 0, 0)
        button.labelcount:SetJustifyH("CENTER")
        button.labelcount:SetPoint("CENTER")

        T.F.mainframe.countbutton = button
    end

    function B.Search.Count:Hide()
        T.F.mainframe.search:Hide()
    end

    function B.Search.Count:Click(...)

        local frame = self:GetParent()

        frame.bank:Hide()
        frame.reagent:Hide()
        frame.character:Hide()
        frame.void:Hide()
        frame.guild:Hide()

        if frame.search:IsVisible() then
            frame.search:Hide()
            if frame.lastbank then frame[frame.lastbank]:Show() end
        else
            frame.search:Show()
        end
    end


    function B.Search:CreateScroll()

        T.F.mainframe.search.enddot = T.F.mainframe.search:CreateTexture(nil, "OVERLAY")
        T.F.mainframe.search.enddot:SetMask("Textures\\MinimapMask")
        T.F.mainframe.search.enddot:SetHeight(5)
        T.F.mainframe.search.enddot:SetWidth(5)
        T.F.mainframe.search.enddot:SetTexture(T.MEDIABLANK)
        T.F.mainframe.search.enddot:SetVertexColor(0.7, 0.7, 0.7, 1)
        T.F.mainframe.search.enddot:SetPoint("TOP", T.F.mainframe.search, "BOTTOM", 0, -5)

        T.F.mainframe.search:SetPoint("TOPLEFT", T.F.mainframe.searchbar, "BOTTOMLEFT", -10, 0)
        T.F.mainframe.search:SetPoint("TOPRIGHT", T.F.mainframe.countbutton, "BOTTOMRIGHT")

        B.I.CreateScroll(T.F.mainframe.search, "FauxScrollFrameBW", B.Search.Update, "BWScrollFrame", self.CreateButtons, 32)
    end

    function B.Search:CreateButtons()
        -- Don't use self here
        local button = B.I.List.Button(index, T.F.mainframe.search.list)

        button:SetScript("OnEnter", B.Search.Enter)
        button:SetScript("OnLeave", B.Search.Leave)
        button:SetScript("OnClick", B.Search.Click)

        local leftstring = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        leftstring:SetJustifyH("LEFT")
        leftstring:SetJustifyV("CENTER")
        leftstring:SetWordWrap(false)
        leftstring:SetPoint("LEFT", button.icon, "RIGHT", 10, 0)
        leftstring:SetPoint("RIGHT", button, "RIGHT", -T.F.mainframe.countbutton:GetWidth(), 0)
        button.leftstring = leftstring

       return button
    end

    function B.Search:Enter()

        if self.quest then
            self:SetBackdropBorderColor(1, 1, 0, 1)
        else
            local r, g, b = GetItemQualityColor(self.quality)
            self:SetBackdropBorderColor(r, g, b, 1)
        end

        B.I.Button.Enter(self, false)
    end

    function B.Search:Leave()

        self:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
        GameTooltip:Hide()
    end

    function B.Search:Click(...)
        local location, page, player, faction, server

        location = self.location.loc
        if location ~= "alt" then
            page = tonumber(self.location.num)
        else
            server, player, faction = self.location.num:match"([^.]*).([^.]*).(.*)"
        end

        if location == "bank" then
            T.F.mainframe.bankbutton:Click()
        elseif location == "reagent" then
            T.F.mainframe.reagentbutton:Click()
        elseif location == "void" then
            T.F.mainframe.voidbutton:Click()
            T.F.mainframe.void[page].tab:Click()
        elseif location == "guild" then
            T.F.mainframe.guildbutton:Click()
            T.F.mainframe.guild[page].tab:Click()
        elseif location == "player" then
            T.F.mainframe.characterbutton:Click()
        elseif location == "alt" then
            B.load(server, player, faction == "H" and "Horde" or faction == "A" and "Alliance" or faction == "N" and "Neutral")
        elseif location == "mail" then
            T.F.mainframe.characterbutton:Click()
            if not T.F.mainframe.character.mail:IsVisible() then T.F.mainframe.character.mail.tab:Click() end
        elseif location == "auctions" then
            T.F.mainframe.characterbutton:Click()
            if not T.F.mainframe.character.auctions:IsVisible() then T.F.mainframe.character.auctions.tab:Click() end
        elseif location == "equip" then
            T.F.mainframe.characterbutton:Click()
            if not T.F.mainframe.character.equipment:IsVisible() then T.F.mainframe.character.tab:Click() end
        end
    end


--[[//////////////////////////////////////////////////

    FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function B.Search:Search(text)

        if T.BW.loaded then
            if text == "" or text == L[9007] then text = false end

            T.F.mainframe.search.scroll.ScrollBar:SetValue(0)
            B.cleantable(T.BW.items)

            B.Bank:Search(text)
            B.Reagent:Search(text)
            B.Void:Search(text)
            B.Guild:Search(text)
            B.Character:Search(text)
            B.Equipment:Search(text)
            B.Mail:Search(text)
            B.Auctions:Search(text)
            self:Alts(text)

            self:Update()

            if text then
                T.F.mainframe.countbutton.labelcount:SetText("|cffffffff"..T.F.mainframe.countbutton.count.."|r")
                T.F.mainframe.countbutton:Show()
                if not T.F.mainframe.search:IsVisible() and (not T.F.mainframe.lastbank or not T.F.mainframe[T.F.mainframe.lastbank]:IsVisible()) then
                    T.F.mainframe.countbutton:Click()
                end
            else
                T.F.mainframe.countbutton:Hide()
            end
        end
    end


    function B.Search:Alts(text)
        local k, v, p, b, player, faction

        if not text then return end

        if BWsavedtable['enablesearchallservers'] then
            for k, v in pairs(BWsavedtable) do
                if not T.OPTIONS[k] then
                    for p, b in pairs(v) do
                        self:Server(k, p, b, text)
                    end
                end
            end
        else -- Only same server and faction as selected player
            for p, b in pairs(BWsavedtable[T.ACTIVE_TABLE.server]) do
                self:Server(T.ACTIVE_TABLE.server, p, b, text)
            end
        end
    end

    local search_item
    function B.Search:Server(server, playerfaction, t, text)
        local player, faction, bank, bags, item, link, info, temp

        search_item = search_item or B.newtable()
        player, faction = playerfaction:match"([^.]*).(.*)"

        if player == T.ACTIVE_TABLE.player and faction == string.sub(T.ACTIVE_TABLE.faction, 1, 1) then return end -- skip loaded character

        if BWsavedtable['enablesearchallfactions'] or faction == string.sub(T.ACTIVE_TABLE.faction, 1, 1) then
            
            for _, bank in pairs(t) do-- banks, bags
                for _, bags in pairs(bank) do -- bag index, bags
                    if type(bags) == "table" then
                        for _, item in pairs(bags) do
                            if type(item) == "table" then

                                link = item['link']
                                info = B.GetItem(link)

                                if next(info) then

                                    search_item['name'] = info[1]
                                    search_item['quality'] = info[3]
                                    search_item['link'] = link
                                    search_item['class'] = info[6]
                                    search_item['subclass'] = info[7]
                                    search_item['equipslot'] = info[9]

                                    if B.find(text, search_item) then
                                        self:AltsSlot(server.."."..playerfaction, link, item['count'] or 1)
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end

    function B.Search:Slot(button)

        button.search:Hide()

        local loc, num = button.loc:match"([^.]*).(.*)"
        num = tonumber(num)

        local result = T.BW.items(loc, num, button.link) -- use metatable __call to group items
        result['count'] = result['count'] + button.count
        T.F.mainframe.countbutton.count = T.F.mainframe.countbutton.count + button.count
    end

    function B.Search:AltsSlot(alt, link, count)

        local result = T.BW.items("alt", alt, link) -- use metatable __call to group items
        result['count'] = result['count'] + count
    end

    function B.Search:Update()

        local r, g, b, fr, fg, fb, idx, link, hexcolor, buttonnum, item
        local player, server, location
        local quality, quest, icon, name, ilvl, faction
        local total_items = #T.BW.items
        local currentscrollbarvalue = FauxScrollFrame_GetOffset(T.F.mainframe.search.scroll)
        local rows = B.RowsToFit(T.F.mainframe.search.list, 32)

        FauxScrollFrame_Update(T.F.mainframe.search.scroll, total_items, rows, 32)
        
        for x = 1, rows do
            -- Clear buttons
            if T.F.mainframe.search.list.buttons[x] then 
                B.ResetButton(T.F.mainframe.search.list.buttons[x])
                --T.F.mainframe.search.list.buttons[x]:SetBackdropColor(T.THEME.r, T.THEME.g, T.THEME.b, 0.7)
                T.F.mainframe.search.list.buttons[x]:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
                --T.F.mainframe.search.list.buttons[x].rightstring.bg:SetVertexColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
                T.F.mainframe.search.list.buttons[x].altstring.faction:Hide()
                T.F.mainframe.search.list.buttons[x].leftstring:SetText("")
                T.F.mainframe.search.list.buttons[x]:Hide()
                T.F.mainframe.search.enddot:Hide()
            end
        end

        for x = 1, rows do
            -- Get the appropriate item, which will be x + value
            if (T.F.mainframe.layout == 2 or T.F.mainframe.layout == 4) and total_items > 0 and total_items < B.RowsToFit(T.F.mainframe.search.list) then
                local difference = B.RowsToFit(T.F.mainframe.search.list) - total_items
                buttonnum = x + difference
            else
                buttonnum = x
            end

            if T.F.mainframe.search.list.buttons[buttonnum] then

                idx = x + currentscrollbarvalue

                if T.BW.items[idx] then
                    link = T.BW.items[idx].link
                    item = B.GetItem(link)
                    name, quality, quest, icon, ilvl = item[1], item[3], (item[6] == L[9006]), item[10], item[4] or nil

                    if quality then
                        r, g, b, hexcolor = GetItemQualityColor(quality)

                        T.F.mainframe.search.list.buttons[buttonnum].quest = quest
                        T.F.mainframe.search.list.buttons[buttonnum].quality = quality
                        T.F.mainframe.search.list.buttons[buttonnum].icon:SetTexture(icon)
                        T.F.mainframe.search.list.buttons[buttonnum].leftstring:SetText("|c"..hexcolor..name)

                        if T.BW.items[idx].count > 1 then
                            T.F.mainframe.search.list.buttons[buttonnum].text:SetText(T.BW.items[idx].count)
                        elseif (item[6] == L[9004] or item[6] == L[9005]) and quality > 1  then
                            ilvl = B.DetectILVL(link) or ilvl
                            T.F.mainframe.search.list.buttons[buttonnum].ilvl:SetText(ilvl)
                            T.F.mainframe.search.list.buttons[buttonnum].ilvl:SetTextColor(r, g, b, 1)
                        end

                        if T.BW.items[idx].loc == "alt" then
                            server, player, faction = T.BW.items[idx].num:match"([^.]*).([^.]*).(.*)"
                            location = player

                            if faction ~= string.sub(T.ACTIVE_TABLE.faction, 1, 1) then
                                if T.ACTIVE_TABLE.faction == "Horde" then
                                    fr, fg, fb = 0.11, 0.44, 0.86
                                else -- Alliance
                                    fr, fg, fb = 0.74, 0.06, 0.19
                                end
                                T.F.mainframe.search.list.buttons[buttonnum].altstring.faction:SetVertexColor(fr, fg, fb, 1)
                                T.F.mainframe.search.list.buttons[buttonnum].altstring.faction:Show()
                            end

                            T.F.mainframe.search.list.buttons[buttonnum].altstring:SetText("|c"..(BWsavedtable[server][player.."."..string.sub(faction, 1, 1)]['player']['class'] and T.RAID_CLASS_COLORS[BWsavedtable[server][player.."."..string.sub(faction, 1, 1)]['player']['class']].colorStr or "ffffffff")..location)
                            T.F.mainframe.search.list.buttons[buttonnum].altstring.server:SetText(server)
                            T.F.mainframe.search.list.buttons[buttonnum].altstring:Show()
                            T.F.mainframe.search.list.buttons[buttonnum].altstring.server:Show()
                            T.F.mainframe.search.list.buttons[buttonnum].rightstring:Hide()
                        else
                            location = T.BW.items[idx].num == 1 and T.LABEL[T.BW.items[idx].loc] or T.LABEL[T.BW.items[idx].loc].." "..T.BW.items[idx].num
                            T.F.mainframe.search.list.buttons[buttonnum].rightstring:SetText(location)
                            T.F.mainframe.search.list.buttons[buttonnum].rightstring:Show()
                            T.F.mainframe.search.list.buttons[buttonnum].altstring:Hide()
                            T.F.mainframe.search.list.buttons[buttonnum].altstring.server:Hide()
                        end
                        T.F.mainframe.search.list.buttons[buttonnum].location = {['loc'] = T.BW.items[idx].loc, ['num'] = T.BW.items[idx].num}
                        T.F.mainframe.search.list.buttons[buttonnum].link = link
                        T.F.mainframe.search.list.buttons[buttonnum]:Show()
                        T.F.mainframe.search.enddot:Show()
                        if idx >= total_items then
                            T.F.mainframe.search.enddot:Hide()
                        end
                    else
                        B.print("Not found "..tostring(link))
                    end
                end
            end
        end
    end


--[[//////////////////////////////////////////////////

    POSITION

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function B.Search:Anchor(startpoint, endpoint, from, to)

        T.F.mainframe.search:ClearAllPoints()
        T.F.mainframe.search:SetPoint(startpoint..from, T.F.mainframe.searchbar, endpoint..from, from == "LEFT" and -10 or 10, 0)
        T.F.mainframe.search:SetPoint(startpoint..to, T.F.mainframe.countbutton, endpoint..to)

        T.F.mainframe.search.enddot:ClearAllPoints()
        T.F.mainframe.search.enddot:SetPoint(startpoint, T.F.mainframe.search, endpoint, 0, startpoint == "TOP" and -5 or 5)

        B.I.Main.Frame.ButtonHighlight(T.F.mainframe.countbutton, endpoint)

        T.F.mainframe.search.list:ClearAllPoints()
        T.F.mainframe.search.list:SetPoint(startpoint..from, T.F.mainframe.search, 0, -2)
        T.F.mainframe.search.list:SetPoint(endpoint..to, T.F.mainframe.search)
    end
