-- GLOBALS: unpack, select, LibStub, ITEM_SEARCHBAR_LIST
-- GLOBALS: VoidStorageFrame, ReagentBankFrame, MerchantFrame

--[[//////////////////////////////////////////////////

    GLO-LO

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    local _G = _G
    local math = math
    local tostring = tostring
    local tonumber = tonumber
    local string = string
    local pairs = pairs
    local format = format
    local tinsert = tinsert
    local strmatch = strmatch
    local strsplit = strsplit

    local CreateFrame = CreateFrame
    local UIParent = UIParent
    local ResetCursor = ResetCursor
    local SetCursor = SetCursor
    local ShowInspectCursor = ShowInspectCursor
    local GameTooltip = GameTooltip
    local GameTooltip_Hide = GameTooltip_Hide
    local GameTooltip_ShowCompareItem = GameTooltip_ShowCompareItem
    local BattlePetToolTip_Show = BattlePetToolTip_Show
    local BattlePetTooltip = BattlePetTooltip
    local CloseDropDownMenus = CloseDropDownMenus
    local ToggleDropDownMenu = ToggleDropDownMenu
    local UIDropDownMenu_CreateInfo = UIDropDownMenu_CreateInfo
    local UIDropDownMenu_SetWidth = UIDropDownMenu_SetWidth
    local UIDropDownMenu_Initialize = UIDropDownMenu_Initialize
    local UIDropDownMenu_AddButton = UIDropDownMenu_AddButton
    local UseContainerItem = UseContainerItem
    local PickupContainerItem = PickupContainerItem
    local PickupGuildBankItem = PickupGuildBankItem
    local GetItemQualityColor = GetItemQualityColor
    local SplitContainerItem = SplitContainerItem
    local SplitGuildBankItem = SplitGuildBankItem
    local ClickVoidStorageSlot = ClickVoidStorageSlot
    local GetCurrentGuildBankTab = GetCurrentGuildBankTab
    local AutoStoreGuildBankItem = AutoStoreGuildBankItem
    local GetCVarBool = GetCVarBool
    local IsShiftKeyDown = IsShiftKeyDown
    local IsModifiedClick = IsModifiedClick
    local IsControlKeyDown = IsControlKeyDown
    local HandleModifiedItemClick = HandleModifiedItemClick
    local SocketContainerItem = SocketContainerItem
    local GetMoneyString = GetMoneyString
    local ColorPickerFrame = ColorPickerFrame
    local FauxScrollFrame_OnVerticalScroll = FauxScrollFrame_OnVerticalScroll
    local IsBagOpen, OpenBag = IsBagOpen, OpenBag
    local IsArtifactPowerItem = IsArtifactPowerItem
    local SocketInventoryItem = SocketInventoryItem
    local SpellCanTargetItem = SpellCanTargetItem
    local SpellCanTargetItemID = SpellCanTargetItemID
    local DressUpItemLink = DressUpItemLink
    local SplitStack = SplitStack
    local ContainerFrameItemButton_OnClick = ContainerFrameItemButton_OnClick
    local StackSplitFrame = StackSplitFrame

    local B, L, T = unpack(select(2, ...))
    B.Void = {}


--[[//////////////////////////////////////////////////

    CREATE FRAME AND BUTTONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function B.Void:CreateFrame()

        local frame = CreateFrame("Frame", "BWvoid", T.F.mainframe, "BackdropTemplate")
        frame:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
        frame:SetFrameStrata("HIGH")
        frame.lasttab = 1

        frame:SetScript("OnShow", B.Void.Show)
        frame:SetScript("OnHide", B.Void.Hide)

        B.I.Backdrop(frame)
        T.F.mainframe.void = frame
    end

    function B.Void:Show()
        T.F.mainframe.voidbutton:LockHighlight()
    end

    function B.Void:Hide()
        T.F.mainframe.voidbutton:UnlockHighlight()
    end


    function B.Void:CreateButton()
        
        local button = B.I.Main.Frame.Button("|cffffffff"..L[10003].."|r")
        button:SetBackdropColor(T.VOIDTAB.r, T.VOIDTAB.g, T.VOIDTAB.b, T.VOIDTAB.a)
        button:SetScript("OnClick", B.Void.Click)

        T.F.mainframe.voidbutton = button
    end

    function B.Void:Click(...)

        local frame = self:GetParent()

        if not T.ISVOIDOPEN then
            frame.character:Hide()
        elseif T.ISVOIDOPEN and frame.character:IsVisible() then
            -- move bags
            B.AnchorBags(frame.void)
        end

        frame.bank:Hide()
        frame.reagent:Hide()
        frame.guild:Hide()
        frame.search:Hide()

        if frame.void:IsVisible() then
            frame.lastbank = nil
            frame.void:Hide()
        else
            frame.lastbank = "void"
            frame.void:Show()
        end
    end


    function B.Void:CreateLayout()

        B.Void:CreateTabs()
        B.Void:CreateButtons()
    end

    function B.Void:CreateTabs()

        local x, lasttab

        for x = 1, T.VOID_PAGES do
            T.F.mainframe.void[x] = {}
            T.F.mainframe.void[x].tab = B.I.Tab.Create(T.F.mainframe.void, nil, "BackdropTemplate")
            T.F.mainframe.void[x].tab.index = x
            T.F.mainframe.void[x].tab:SetScript("OnClick", B.Void.ClickTab)

            if x == 1 then
                T.F.mainframe.void[x].tab:SetBackdropBorderColor(1, 1, 1, 1)
                T.F.mainframe.void[x].tab:SetNormalTexture("Interface\\Icons\\INV_Enchant_EssenceCosmicGreater")
                T.F.mainframe.void[x].tab:SetPoint("TOPLEFT", T.F.mainframe.void, "TOPRIGHT")
            else
                T.F.mainframe.void[x].tab:SetNormalTexture("Interface\\Icons\\INV_Enchant_EssenceArcaneLarge")
                T.F.mainframe.void[x].tab:SetPoint("TOPLEFT", lasttab, "BOTTOMLEFT", 0, -4)
            end
            lasttab = T.F.mainframe.void[x].tab
        end
    end

    function B.Void:ClickTab(...)

        local parent = self:GetParent()

        parent[parent.lasttab].frame:Hide()
        parent[parent.lasttab].tab:SetBackdropBorderColor(0, 0, 0, 1)

        self:SetBackdropBorderColor(1, 1, 1, 1)
        parent[self.index].frame:Show()
        parent.lasttab = self.index

        if T.ISVOIDOPEN and VoidStorage_SetPageNumber then
            VoidStorage_SetPageNumber(self.index)
        end
    end

    function B.Void:CreateButtons()

        local x, b, button, rows

        for x = 1, T.VOID_PAGES do
            B.I.Frame.Create(T.F.mainframe.void, T.F.mainframe.void[x], x)
            T.F.mainframe.void[x].frame:SetID(x)

            for b = 1, T.NUM_VOID_SLOTS do
                button = B.I.Button.Create(b, T.F.mainframe.void[x].frame)
                button:SetScript("OnEnter", B.I.Button.Enter)
                button:SetScript("OnLeave", B.I.Button.Leave)
                T.F.mainframe.void[x].buttons[b] = button
            end

            rows = B.I.Grid.Update(T.F.mainframe.void[x])
        end

        T.F.mainframe.void:SetHeight(10 + (T.BUTTON_HEIGHT * (rows + 1)) + (T.FRAMEWHITESPACE * rows))
        T.F.mainframe.void:SetWidth(20 + (T.BUTTON_HEIGHT * T.ROW) + (T.FRAMEWHITESPACE * (T.ROW - 1)))
    end


--[[//////////////////////////////////////////////////

    FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function B.Void:IsSlot(button)
        local name = button:GetName()
        if name and strfind(name, "VoidStorageStorageButton%d+") then
            local id = strmatch(name, ("%d+"))
            button:SetID(tonumber(id)) -- Blizzard doesn't set id, so I will
            return true
        end
        return false
    end

    function B.Void:Scan()
        -- This is used to force scan the entire bank in the event the user is using a bag addon that
        -- modifies the blizzard frame and makes it impossible to scan.
        if not IsVoidStorageReady() then return end
        local x, page, itemID
        local void = T['void']

        for page = 1, T.VOID_PAGES do
            if not void[page] then void[page] = B.newtable() end

            for x = 1, T.NUM_VOID_SLOTS do
                itemID = GetVoidItemInfo(page, x)

                if itemID then
                    void[page][x] = void[page][x] or B.newtable()
                    void[page][x]['link'] = page == 1 and GetVoidItemHyperlinkString(x) or GetVoidItemHyperlinkString(T.NUM_VOID_SLOTS + x)
                else
                    void[page][x] = B.deltable(void[page][x])
                end
            end
        end

        if B.IsCurrentPlayer() then self:Update() end
        B.CacheMainItems('void')
    end

    function B.Void:ScanSlot(page, slot, button)

        local void = T['void']
        local link = nil
        if button.hasItem then link = page == 1 and GetVoidItemHyperlinkString(slot) or GetVoidItemHyperlinkString(T.NUM_VOID_SLOTS + slot) end

        if not void[page] then void[page] = B.newtable() end

        if not B.CompareSlot(void[page][slot], link) then

            void[page][slot] = B.Scan(void[page][slot], link)
            if B.IsCurrentPlayer() then self:UpdateSlot(page, slot) end
            B.CacheMainItems('void')
        end
    end

    function B.Void:Update()

        local page, void = _, B.GetPlayerBank('void')

        for page = 1, T.VOID_PAGES do
            B.UpdatePage(T.F.mainframe.void[page], T.NUM_VOID_SLOTS, void[page])
        end
    end

    function B.Void:UpdateSlot(page, slot)
        local void = B.GetPlayerBank('void')
        B.UpdateSlot(T.F.mainframe.void[page].buttons[slot], void[page][slot])
    end

    function B.Void:Search(text)

        local x, page, button, buttons
        local void = B.GetPlayerBank('void')

        for page = 1, T.VOID_PAGES do
            if text then T.F.mainframe.void[page].tab.search:Show() 
            else T.F.mainframe.void[page].tab.search:Hide() end

            buttons = T.F.mainframe.void[page].buttons

            for x = 1, T.NUM_VOID_SLOTS do
                button = buttons[x]

                if button.name and text then
                    if B.find(text, button) then
                        button.loc = "void."..page
                        B.Search:Slot(button)
                        T.F.mainframe.void[page].tab.search:Hide()
                    else
                        button.search:Show()
                    end
                else
                    button.search:Hide()
                end
            end
        end
    end


--[[//////////////////////////////////////////////////

    POSITION

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function B.Void:Anchor(startpoint, endpoint, from, to)

        local lasttab

        T.F.mainframe.void:ClearAllPoints()
        T.F.mainframe.void:SetPoint(startpoint..from, T.F.mainframe, endpoint..from)

        B.I.Main.Frame.ButtonHighlight(T.F.mainframe.voidbutton, endpoint)

        for x = 1, T.VOID_PAGES do
            T.F.mainframe.void[x].tab:ClearAllPoints()
            if x == 1 then
                T.F.mainframe.void[x].tab:SetPoint("TOP"..from, T.F.mainframe.void, "TOP"..to)
            else
                T.F.mainframe.void[x].tab:SetPoint("TOP"..from, lasttab, "BOTTOM"..from, 0, -4)
            end
            
            lasttab = T.F.mainframe.void[x].tab
        end
    end
