-- GLOBALS: unpack, select, LibStub

--[[//////////////////////////////////////////////////

    GLO-LO

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    local _
    local _G = _G
    local next = next
    local type = type
    local pairs = pairs
    local math = math
    local tonumber = tonumber
    local string = string
    local setmetatable = setmetatable
    local tostring = tostring
    local strfind = strfind
    local strlower = strlower
    local strmatch = strmatch
    local strsplit = strsplit
    local format = format
    local floor = floor
    local SecondsToTime = SecondsToTime

    local DEFAULT_CHAT_FRAME = DEFAULT_CHAT_FRAME
    local UIParent = UIParent
    local UnitName = UnitName
    local UnitClass = UnitClass
    local GetRealmName = GetRealmName
    local UnitFactionGroup = UnitFactionGroup
    local GetScreenWidth = GetScreenWidth
    local GetScreenHeight = GetScreenHeight
    local GetCursorPosition = GetCursorPosition
    local GetCoinTextureString = GetCoinTextureString
    local GetMoney = GetMoney
    local IsArtifactPowerItem = IsArtifactPowerItem
    local C_PetJournal = C_PetJournal

    local B, L, T = unpack(select(2, ...))
    local Search = LibStub:GetLibrary('LibItemSearch-1.0')

    local ilvl_pattern = _G['ITEM_LEVEL']:gsub("%%d", "(%%d+)")
    local tooltip = CreateFrame("GameTooltip", "BW_tooltip", nil, "GameTooltipTemplate")
    tooltip:SetOwner(UIParent, "ANCHOR_NONE")

--[[//////////////////////////////////////////////////

    HELPER FUNCTIONS

    Anything that needs repetitive actions or calculations
    Tips: 
        -- Multiplication are faster than division
        -- Local is faster than global. Re-assign global as a new local if used a lot

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    -- Table pool for recycling tables
    local tables = setmetatable({}, {__mode = "kv"}) -- Weak table

    -- Get a new table
    function B.newtable()
        local t = next(tables) or {}

        tables[t] = nil
        return t
    end

    function B.newsearchtable()
        local t = B.newtable()

        setmetatable(t, {
        __call = function(t, loc, num, link)
            local x

            for x = 1, #t do
                if t[x].loc == loc and t[x].num == num and t[x].link == link then
                    return t[x]
                end
            end

            t[#t + 1] = {['loc'] = loc, ['num'] = num, ['link'] = link, ['count'] = 0}
            return t[#t]
        end
        })
        return t
    end

    function B.cleantable(t)
        local x

        for x = 1, #t do
            t[x] = nil
        end
    end

    -- Delete table and add to table pool
    function B.deltable(t)
        local k, v

        if type(t) == "table" then
            for k, v in pairs(t) do
                if type(v) == "table" then B.deltable(v) end
                t[k] = nil
            end
            
            setmetatable(t, nil)
            t[true] = true
            t[true] = nil
            tables[t] = true
        end
        return nil
    end

    -- tcopy: recursively copy contents of one table to another.  from wowwiki
    function B.copytable(to, from)
        -- "to" must be a table (possibly empty)
        for k, v in pairs(from) do

            if type(v) == "table" then
                to[k] = {}
                B.copytable(to[k], v)
            else
                to[k] = v
            end
        end
    end

    function B.strstart(string, start)
       return string.sub(string, 1, string.len(start)) == start
    end

    function B.hex(r, g, b)
        return string.format("|cff%02x%02x%02x", (r or 1) * 255, (g or 1) * 255, (b or 1) * 255)
    end

    ------ GOLD MANAGEMENT

    local ASGetGSC = function (money)
            if (money == nil) then money = 0 end
            local g = math.floor(money / 10000)
            local s = math.floor((money - (g*10000)) / 100)
            local c = math.ceil(money - (g*10000) - (s*100))
            return g,s,c
    end

    -- formats money text by color for gold, silver, copper
    function B.ASGSC(money, exact, dontUseColorCodes, icon, letters, size)
        -------------- THANK YOU BOTTOMSCANNER ----------------
        --if not (exact) then exact = true end;
        if (type(money) ~= "number") then return end
        if (not money) then money = 0 end
        if (not exact) and (money >= 10000) then money = math.floor(money / 100 + 0.5) * 100 end

        if icon == false then
            local TEXT_NONE = "0"
               
            local GSC_GOLD="ffd100"
            local GSC_SILVER="e0e0e0"
            local GSC_COPPER="c8602c"
            local GSC_VALUE = "ffffff"
            local GSC_START="|cff%s%s|r|cff%s%s|r"
            local GSC_PART=".|cff%s%s|r|cff%s%s|r"
            local GSC_NONE="|cffa0a0a0"..TEXT_NONE.."|r"
           
            local g, s, c = ASGetGSC(money)
           
            local gsc = ""
            local fmt = GSC_START
            if (g > 0) then gsc = gsc..string.format(fmt, GSC_GOLD, g, GSC_GOLD, letters and "g" or "") fmt = GSC_PART end
            if (s > 0) or (c > 0) then gsc = gsc..string.format(fmt, GSC_SILVER, s, GSC_SILVER, letters and "s" or "") fmt = GSC_PART end
            if (c > 0) then gsc = gsc..string.format(fmt, GSC_COPPER, c, GSC_COPPER, letters and "c" or "") end
            if (gsc == "") then gsc = GSC_NONE end

            return gsc
        else
            return "|cffffffff"..GetCoinTextureString(money, size or 11).."|r"--gsc
        end
    end

    function B.round(total)

        return 100 * math.floor((total + 50) / 100)
    end

    function B.ValidateConfigs()
        if not BWsavedtable then BWsavedtable = {} end

        local BWsavedtable = BWsavedtable
        BWsavedtable['iconsizeslider'] = BWsavedtable['iconsizeslider'] or 32
        BWsavedtable['iconsize'] = BWsavedtable['iconsize'] or 32
        BWsavedtable['iconrowslider'] = BWsavedtable['iconrowslider'] or 12
        BWsavedtable['iconrow'] = BWsavedtable['iconrow'] or 12
        BWsavedtable['minimize'] = BWsavedtable['minimize'] or false
        BWsavedtable['bankbagstoggle'] = BWsavedtable['bankbagstoggle'] == true and true or BWsavedtable['bankbagstoggle'] == nil and true or false
        BWsavedtable['bankbagtooltiptoggle'] = BWsavedtable['bankbagtooltiptoggle'] == true and true or BWsavedtable['bankbagtooltiptoggle'] == nil and true or false
        BWsavedtable['playerbagtooltiptoggle'] = BWsavedtable['playerbagtooltiptoggle'] == true and true or BWsavedtable['playerbagtooltiptoggle'] == nil and true or false
        BWsavedtable['playerbagstoggle'] = BWsavedtable['playerbagstoggle'] == true and true or BWsavedtable['playerbagstoggle'] == nil and true or false
        BWsavedtable['enablesoundsmoneytracker'] = BWsavedtable['enablesoundsmoneytracker'] == true or BWsavedtable['enablesoundsmoneytracker'] == nil and true or false
        BWsavedtable['enablemoneytracker'] = BWsavedtable['enablemoneytracker'] == true and true or BWsavedtable['enablemoneytracker'] == nil and true or false
        BWsavedtable['moneysound'] = BWsavedtable['moneysound'] or 2
        BWsavedtable['treasuresound'] = BWsavedtable['treasuresound'] or 3
        BWsavedtable['treasuresoundtracker'] = BWsavedtable['treasuresoundtracker'] or 1000000
        BWsavedtable['minimumsoundtracker'] = BWsavedtable['minimumsoundtracker']
        BWsavedtable['scalemoneytracker'] = BWsavedtable['scalemoneytracker'] or 125
        BWsavedtable['colorbank'] = BWsavedtable['colorbank']
        BWsavedtable['colorbanklabel'] = BWsavedtable['colorbanklabel']
        BWsavedtable['colorreagent'] = BWsavedtable['colorreagent']
        BWsavedtable['colorreagentlabel'] = BWsavedtable['colorreagentlabel']
        BWsavedtable['colorcharacter'] = BWsavedtable['colorcharacter']
        BWsavedtable['colorcharacterlabel'] = BWsavedtable['colorcharacterlabel']
        BWsavedtable['colorvoid'] = BWsavedtable['colorvoid']
        BWsavedtable['colorvoidlabel'] = BWsavedtable['colorvoidlabel']
        BWsavedtable['colorguild'] = BWsavedtable['colorguild']
        BWsavedtable['colorguildlabel'] = BWsavedtable['colorguildlabel']
        BWsavedtable['colortooltip'] = BWsavedtable['colortooltip']
        BWsavedtable['colortooltipaccent'] = BWsavedtable['colortooltipaccent']
        BWsavedtable['minimapicon'] = BWsavedtable['minimapicon'] or { hide = false }

        if not BWconfig then
            BWconfig = {
                ['iconsizeslider'] = BWsavedtable['iconsizeslider'] or 32,
                ['iconsize'] = BWsavedtable['iconsize'] or 32,
                ['iconrowslider'] = BWsavedtable['iconrowslider'] or 12,
                ['iconrow'] = BWsavedtable['iconrow'] or 12,
                ['minimize'] = BWsavedtable['minimize'] or false,
                ['bankbagstoggle'] = BWsavedtable['bankbagstoggle'] == true and true or BWsavedtable['bankbagstoggle'] == nil and true or false,
                ['bankbagtooltiptoggle'] = BWsavedtable['bankbagtooltiptoggle'] == true and true or BWsavedtable['bankbagtooltiptoggle'] == nil and true or false,
                ['playerbagtooltiptoggle'] = BWsavedtable['playerbagtooltiptoggle'] == true and true or BWsavedtable['playerbagtooltiptoggle'] == nil and true or false,
                ['playerbagstoggle'] = BWsavedtable['playerbagstoggle'] == true and true or BWsavedtable['playerbagstoggle'] == nil and true or false,
                ['enablesoundsmoneytracker'] = BWsavedtable['enablesoundsmoneytracker'] == true or BWsavedtable['enablesoundsmoneytracker'] == nil and true or false,
                ['enablemoneytracker'] = BWsavedtable['enablemoneytracker'] == true and true or BWsavedtable['enablemoneytracker'] == nil and true or false,
                ['moneysound'] = BWsavedtable['moneysound'] or 2,
                ['treasuresound'] = BWsavedtable['treasuresound'] or 3,
                ['treasuresoundtracker'] = BWsavedtable['treasuresoundtracker'] or 1000000,
                ['minimumsoundtracker'] = BWsavedtable['minimumsoundtracker'],
                ['scalemoneytracker'] = BWsavedtable['scalemoneytracker'] or 125,
                ['colorbank'] = BWsavedtable['colorbank'],
                ['colorbanklabel'] = BWsavedtable['colorbanklabel'],
                ['colorreagent'] = BWsavedtable['colorreagent'],
                ['colorreagentlabel'] = BWsavedtable['colorreagentlabel'],
                ['colorcharacter'] = BWsavedtable['colorcharacter'],
                ['colorcharacterlabel'] = BWsavedtable['colorcharacterlabel'],
                ['colorvoid'] = BWsavedtable['colorvoid'],
                ['colorvoidlabel'] = BWsavedtable['colorvoidlabel'],
                ['colorguild'] = BWsavedtable['colorguild'],
                ['colorguildlabel'] = BWsavedtable['colorguildlabel'],
                ['colortooltip'] = BWsavedtable['colortooltip'],
                ['colortooltipaccent'] = BWsavedtable['colortooltipaccent'],
                ['perconfig'] = false
            }
        end

        BWconfig['state'] = BWconfig['state'] or 0
    end

    function B.template(server, player, faction)

        T.ACTIVE_TABLE = {['server'] = server, ['player'] = player, ['faction'] = faction}
        B.ValidateConfigs()

        local BWsavedtable = BWsavedtable
        if not BWsavedtable[server] then BWsavedtable[server] = {} end

        BWsavedtable[server][player.."."..string.sub(faction, 1, 1)] = {['bank'] = {{['slots'] = T.NUM_BANKGENERIC_SLOTS}}, ['reagent'] = {{}},
                                                                        ['void'] = {{}, {}}, ['guild'] = {}, ['player'] = {[0] = {['slots'] = T.NUM_BAG_SLOTS, ['class'] = false}, [-1] = {}},
                                                                        ['auctions'] = {{}}, ['mail'] = {{}}, ['currency'] = {['money'] = 0}}
        B.load(server, player, faction)
    end

    function B.load(server, player, faction)
        local text, playerfaction
        local BWsavedtable = BWsavedtable

        T.ACTIVE_TABLE = {['server'] = server, ['player'] = player, ['faction'] = faction}
        playerfaction = player.."."..string.sub(faction, 1, 1)
        -- Validate settings added at a later time
        if not BWsavedtable[server][playerfaction]['player'] then BWsavedtable[server][playerfaction]['player'] = {[0] = {['slots'] = T.NUM_BAG_SLOTS, ['class'] = false}, [-1] = {}} end
        if not BWsavedtable[server][playerfaction]['player']['class'] then BWsavedtable[server][playerfaction]['player']['class'] = false end
        if not BWsavedtable[server][playerfaction]['auctions'] then BWsavedtable[server][playerfaction]['auctions'] = {{}} end
        if not BWsavedtable[server][playerfaction]['mail'] then BWsavedtable[server][playerfaction]['mail'] = {{}} end
        if not BWsavedtable[server][playerfaction]['currency'] then BWsavedtable[server][playerfaction]['currency'] = {['money'] = 0} end

        B.cleantable(T.BW.items)

        if B.IsCurrentPlayer() then
            T.F.mainframe.alt = nil
            -- Reference tables for WoW loaded character
            T['bank'] = B.GetPlayerBank('bank', true)
            T['reagent'] = B.GetPlayerBank('reagent', true)
            T['void'] = B.GetPlayerBank('void', true)
            T['guild'] = B.GetPlayerBank('guild', true)
            T['player'] = B.GetPlayerBank('player', true)
            T['auctions'] = B.GetPlayerBank('auctions', true)
            T['mail'] = B.GetPlayerBank('mail', true)
            T['currency'] = B.GetPlayerBank('currency', true)
            T['currency']['money'] = GetMoney()

            BWsavedtable[server][playerfaction]['player']['class'] = select(2, UnitClass('player'))
            T['class'] = T['player']['class']
        else
            T.F.mainframe.alt = true
        end

        T.CLASS = T.RAID_CLASS_COLORS[B.GetPlayerBank('player')['class'] or T['class']]
        T.F.status:SetVertexColor(T.CLASS.r, T.CLASS.g, T.CLASS.b, 1)

        --B.UpdateBanks()
    end

    function B.IsCurrentPlayer()

        local player = UnitName("player")
        local server = GetRealmName()
        local faction = UnitFactionGroup("player")

        if T.ACTIVE_TABLE.server == server and T.ACTIVE_TABLE.player == player and T.ACTIVE_TABLE.faction then
            return true
        end
        return false
    end

    function B.GetPlayerBank(bank, current)
        local BWsavedtable = BWsavedtable

        if not bank then return end
        if current then
            local player = UnitName("player")
            local server = GetRealmName()
            local faction = UnitFactionGroup("player")
            return BWsavedtable[server][player.."."..string.sub(faction, 1, 1)][bank]
        end

        return BWsavedtable[T.ACTIVE_TABLE.server][T.ACTIVE_TABLE.player.."."..string.sub(T.ACTIVE_TABLE.faction, 1, 1)][bank]
    end


    local stack = -1
    local messagestr = ""

    function B.print(message, level)
        local filler

        if BWdebug or (level == 1) then

            if stack == -1 then messagestr = T.MSGC.DEFAULT.."Bank's Watch|r: " end

            if not message then
                DEFAULT_CHAT_FRAME:AddMessage(messagestr..tostring(message))
                return false 
            end

            stack = stack + 1

            if type(message) == "table" then
                
                filler = string.rep(". . ", stack)
                DEFAULT_CHAT_FRAME:AddMessage(messagestr.."{table} -->")
                for k, v in pairs(message) do
                    messagestr = filler.."["..k.."] = "
                    B.print(v)
                end
            
            elseif type(message) == "userdata" then

            elseif type(message) == "function" then
                DEFAULT_CHAT_FRAME:AddMessage(messagestr.." A function(self)")
            elseif type(message) == "boolean" then
                DEFAULT_CHAT_FRAME:AddMessage(messagestr..tostring(message))
            else
                DEFAULT_CHAT_FRAME:AddMessage(messagestr..tostring(message))
            end
            
            stack = stack - 1
       end
    end

    function B.find(text, itemOrButton)

        text = text:gsub("[%[%]%(%)]", "") --remove brackets it throws error in pattern matching...
        if text == "" then return end

        if strfind(strlower(L[9010]), text) and IsArtifactPowerItem(B.GetItemIdFromLink(itemOrButton.link)) then
            return true
        elseif strfind(strlower(L[9013]), text) and strfind(itemOrButton.link, "|Hbattlepet:") then
            return true
        elseif strfind(text, "|Hbattlepet:") then -- isolate battle pet link search
            if B.GetPetInfo(itemOrButton.link) == B.GetPetInfo(text) then
                return true
            end
        elseif strfind(text, "|Hitem:") then --isolate itemlink search
            if B.GetItemIdFromLink(itemOrButton.link) == B.GetItemIdFromLink(text) then
                return true
            end
        else
            return Search:Find(itemOrButton, text)
        end
    end

    function B.CompareSlot(slot, link, count)

        if (slot and slot['link'] == link and ((slot['count'] and slot['count'] == count) or ((count == 1 or not count) and not slot['count'])) or
           (not slot and not link)) then -- same
           return true
        end

        return false
    end

    function B.GetCursorScaledPosition()
        local scale, x, y = UIParent:GetScale(), GetCursorPosition()
        return x / scale, y / scale
    end

    function B.SetFrameLocation(forceRefresh)

        local x, y = T.F.mainframe.optionbutton:GetRect()
        -- Only hide when cursor exits at the top or bottom of the option menu
        if (x < (GetScreenWidth()/2)) and (y > (GetScreenHeight()/2)) then
            if forceRefresh or T.F.mainframe.layout ~= 1 then B.I.AnchorBagsTopLeft() end
        elseif (x < (GetScreenWidth()/2)) and (y < (GetScreenHeight()/2)) then
            if forceRefresh or T.F.mainframe.layout ~= 2 then B.I.AnchorBagsBottomLeft() end
        elseif (x > (GetScreenWidth()/2)) and (y > (GetScreenHeight()/2)) then
            if forceRefresh or T.F.mainframe.layout ~= 3 then B.I.AnchorBagsTopRight() end
        elseif (x > (GetScreenWidth()/2)) and (y < (GetScreenHeight()/2)) then
            if forceRefresh or T.F.mainframe.layout ~= 4 then B.I.AnchorBagsBottomRight() end
        end

        B.Search:Update()
        B.Auctions:Update()
        B.Mail:Update()
    end

    function B.RowsToFit(list, height)

        local ourheight = math.ceil(list:GetHeight()) - T.FRAMEWHITESPACE
        return math.floor(ourheight / (height or T.BUTTON_HEIGHT))
    end

    function B.HideAll()

        T.F.mainframe.lastbank = nil
        T.F.mainframe.bank:Hide()
        T.F.mainframe.reagent:Hide()
        T.F.mainframe.void:Hide()
        T.F.mainframe.guild:Hide()
        T.F.mainframe.search:Hide()
        T.F.mainframe.character:Hide()
    end

    function B.DetectILVL(itemlink)
        local text, ilvl

        tooltip:SetHyperlink(itemlink)

        text = _G['BW_tooltipTextLeft2']:GetText()
        if text then ilvl = tonumber(text:match(ilvl_pattern)) end

        tooltip:ClearLines()
        return ilvl or nil
    end

    function B.DetectAP(bag, slot)
        local text, ap, k

        tooltip:SetOwner(UIParent, "ANCHOR_NONE")
        tooltip:SetBagItem(bag, slot)

        for k = tooltip:NumLines(), 2, -1 do
            ap = (_G["BW_tooltipTextLeft"..k]:GetText() or ""):match("(%d*%.?%d+ %a+ "..L[9010]..")")
            if ap then
                ap = ap:gsub(",", "")
                tooltip:ClearLines()
                return ap
            end
        end

        tooltip:ClearLines()
        return 0
    end

    function B.GetGuildPetLink(page, slot)
        local speciesID, level, breedQuality, maxHealth, power, speed, name = tooltip:SetGuildBankItem(page, slot)
        local link = string.format("%s|Hbattlepet:%s:%s:%s:%s:%s:%s:|h[%s]|h|r", T.ITEM_QUALITY_COLORS[breedQuality].hex, speciesID, level, breedQuality, maxHealth, power, speed, name)
        tooltip:ClearLines()

        return link
    end

    function B.GetPetInfo(itemlink)
        local speciesID, level, breedQuality, maxHealth, power, speed, name, icon

        _, speciesID, level, breedQuality, maxHealth, power, speed, name = strsplit(":", itemlink)
        name, icon = C_PetJournal.GetPetInfoBySpeciesID(speciesID)
        return name, itemlink, tonumber(breedQuality), nil, nil, nil, nil, nil, nil, icon -- Keep same structure as getiteminfo
    end

    function B.GetKeystoneInfo(itemlink)
        local _, _, _, _, _, _, _, _, _, _, _, _, _, _, name = string.find(itemlink, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?")
        return name, itemlink, 4, nil, nil, nil, nil, nil, nil, 525134 -- Keep same structure as getiteminfo
    end

    function B.GetItemIdFromLink(itemlink)
        local itemstring = strmatch(itemlink, "item[%-?%d:]+")
        if not itemstring then return nil end
        local _, itemid = strsplit(":", itemstring)

        return itemid
    end

    function B.GetQualityFromLink(itemlink)
        local _, _, color = strfind(itemlink, "|?c?f?f?(%x*)|")
        return T.QUALITY[color]
    end

    function B.GetBag(button)
        return button:GetParent():GetID(), button:GetID()
    end

    function B.GetBankBag(button)
        return button:GetParent():GetID(), button:GetID()
    end

    function B.GetSlotID(bag, slot)
        local total, lasttotal, page
        local bank = T['bank']

        total = 0

        for page = 1, T.NUM_BAG_SLOTS + T.NUM_BANKBAGSLOTS do
            if bank[page] then
                
                if page == bag then return total + slot end
                total = total + bank[page]['slots']
            end
        end
    end

    function B.GetMailIcon(total)
        -- return mail texture
        if not total or total == 0 then total = 0 end

        if total < 10 then return 133468
        elseif total < 15 then return 133461
        elseif total < 25 then return 133467
        elseif total < 45 then return 133470
        else return 133462
        end
    end

    function B.GetDays(daysLeft)

        if ( daysLeft >= 1 ) then
            return "|cff20ff20"..format(L[9014], floor(daysLeft)).."|r"
        else
            return "|cffff2020"..SecondsToTime(floor(daysLeft * 24 * 60 * 60)).."|r"
        end
    end
