-- GLOBALS: unpack, select, LibStub, ITEM_SEARCHBAR_LIST
-- GLOBALS: VoidStorageFrame, ReagentBankFrame, MerchantFrame

--[[//////////////////////////////////////////////////

    GLO-LO

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    local _G = _G
    local math = math
    local tostring = tostring
    local tonumber = tonumber
    local string = string
    local pairs = pairs
    local format = format
    local tinsert = tinsert
    local strmatch = strmatch
    local strsplit = strsplit

    local CreateFrame = CreateFrame
    local UIParent = UIParent
    local ResetCursor = ResetCursor
    local SetCursor = SetCursor
    local ShowInspectCursor = ShowInspectCursor
    local GameTooltip = GameTooltip
    local GameTooltip_Hide = GameTooltip_Hide
    local GameTooltip_ShowCompareItem = GameTooltip_ShowCompareItem
    local BattlePetToolTip_Show = BattlePetToolTip_Show
    local BattlePetTooltip = BattlePetTooltip
    local CloseDropDownMenus = CloseDropDownMenus
    local ToggleDropDownMenu = ToggleDropDownMenu
    local UIDropDownMenu_CreateInfo = UIDropDownMenu_CreateInfo
    local UIDropDownMenu_SetWidth = UIDropDownMenu_SetWidth
    local UIDropDownMenu_Initialize = UIDropDownMenu_Initialize
    local UIDropDownMenu_AddButton = UIDropDownMenu_AddButton
    local UseContainerItem = UseContainerItem
    local PickupContainerItem = PickupContainerItem
    local PickupGuildBankItem = PickupGuildBankItem
    local GetItemQualityColor = GetItemQualityColor
    local SplitContainerItem = SplitContainerItem
    local SplitGuildBankItem = SplitGuildBankItem
    local ClickVoidStorageSlot = ClickVoidStorageSlot
    local GetCurrentGuildBankTab = GetCurrentGuildBankTab
    local AutoStoreGuildBankItem = AutoStoreGuildBankItem
    local GetCVarBool = GetCVarBool
    local IsShiftKeyDown = IsShiftKeyDown
    local IsModifiedClick = IsModifiedClick
    local IsControlKeyDown = IsControlKeyDown
    local HandleModifiedItemClick = HandleModifiedItemClick
    local SocketContainerItem = SocketContainerItem
    local GetMoneyString = GetMoneyString
    local ColorPickerFrame = ColorPickerFrame
    local FauxScrollFrame_OnVerticalScroll = FauxScrollFrame_OnVerticalScroll
    local IsBagOpen, OpenBag = IsBagOpen, OpenBag
    local IsArtifactPowerItem = IsArtifactPowerItem
    local SocketInventoryItem = SocketInventoryItem
    local SpellCanTargetItem = SpellCanTargetItem
    local SpellCanTargetItemID = SpellCanTargetItemID
    local DressUpItemLink = DressUpItemLink
    local SplitStack = SplitStack
    local ContainerFrameItemButton_OnClick = ContainerFrameItemButton_OnClick
    local StackSplitFrame = StackSplitFrame

    local B, L, T = unpack(select(2, ...))
    local LibDD = LibStub:GetLibrary("LibUIDropDownMenu-4.0")

    -- forward declaration
    local create_bank, create_reagent, create_character, create_void, create_guild, create_search

    B.I =  {['bank'] = {['num'] = 0, ['rows'] = 0},
            ['character'] = {['num'] = 0, ['rows'] = 0},
            ['OptionsDropDown'] = {},
            ['Bank'] = {['Bags'] = {}, ['Buttons'] = {}, ['Tooltip'] = {}, ['BagToggle'] = {}, ['DefaultToggle'] = {}},
            ['Character'] = {['Bags'] = {}, ['DefaultToggle'] = {}, ['Buttons'] = {}, ['Tab'] = {}, ['Tooltip'] = {}, ['BagToggle'] = {},
                             ['Artifact'] = {}, ['Equipment'] = {}, ['Auctions'] = {['Button'] = {}},
                             ['Mail'] = {['Button'] = {}}, ['Farm'] = {['Button'] = {}}},
            ['Reagent'] = {},
            ['Void'] = {['Tabs'] = {}},
            ['Guild'] = {['Tabs'] = {}, ['DefaultToggle'] = {}},
            ['Search'] = {['Count'] = {}, ['Button'] = {}},
            ['Toggle'] = {},
            ['Options'] = {},
            ['Main'] = {['Frame'] = {}}
            }


--[[//////////////////////////////////////////////////

    MAIN FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    function B.CreateFrames()

        B.Toggle:CreateFrame()
        B.Base:CreateFrame()

        B.Bank:CreateFrame()
        B.Reagent:CreateFrame()
        B.Character:CreateFrame()
        B.Void:CreateFrame()
        B.Guild:CreateFrame()
        B.Search:CreateFrame()
    end

    function B.CreateLayouts()

        B.Bank:CreateLayout()
        B.Reagent:CreateLayout()
        B.Character:CreateLayout()
        B.Void:CreateLayout()
        B.Guild:CreateLayout()
        B.Search:CreateLayout()

        if not T.CONFIG['bankbagstoggle'] then T.F.mainframe.bank.bags:Hide() end
        if not T.CONFIG['playerbagstoggle'] then T.F.mainframe.character.bags:Hide() end
    end

--[[//////////////////////////////////////////////////

    MAINFRAME FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]


    function B.I.Main.Frame.Button(label, parent, name)
        local button = CreateFrame("Button", name, parent or T.F.mainframe, "BackdropTemplate")

        button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
        button:SetWidth(30)
        button:SetHeight(T.F.mainframe:GetHeight())
        button:SetNormalFontObject("GameFontNormal")
        button:SetText(label)
        button:SetBackdrop({bgFile = T.MEDIABLANK,  edgeFile = T.MEDIABLANK, edgeSize = 1})
        button:SetBackdropBorderColor(0, 0, 0, 0)

        local highlight = button:CreateTexture(nil, "HIGHLIGHT")
        highlight:SetTexture(T.MEDIABLANK)
        highlight:SetHeight(2)
        highlight:SetVertexColor(1, 1, 1, 1)
        button:SetHighlightTexture(highlight)
        B.I.Main.Frame.ButtonHighlight(button, "BOTTOM")

        return button
    end

    function B.I.Main.Frame.OptionsButton(text, parent, name, inherit)

        local button = CreateFrame("Button", name, parent or T.F.mainframe, inherit)
        button:SetHeight(12)
        button:SetWidth(50)

        B.I.Backdrop(button)

        local label = button:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
        label:SetTextColor(1, 1, 1)
        label:SetAllPoints()
        label:SetFont(label:GetFont(), 7)
        label:SetJustifyH("CENTER")
        label:SetJustifyV("CENTER")
        label:SetWordWrap(false)
        label:SetText(text)
        button.label = label

        button:HookScript("OnEnter", function(self) self:SetBackdropBorderColor(1, 1, 1, 1) end)
        button:HookScript("OnLeave", function(self) self:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1) end)

        return button
    end

    function B.I.Create_FrameButton(label, frame, scripts)

        local frame = B.I.Main.Frame.OptionsButton(label, frame, nil, "BackdropTemplate")
        frame:HookScript("OnClick", scripts.Click)
        frame:HookScript("OnEnter", scripts.Enter)
        frame:HookScript("OnLeave", scripts.Leave)

        return frame
    end

    local function create_framebutton(label, frame, scripts)

        local frame = B.I.Main.Frame.OptionsButton(label, frame, nil, "BackdropTemplate")
        frame:HookScript("OnClick", scripts.Click)
        frame:HookScript("OnEnter", scripts.Enter)
        frame:HookScript("OnLeave", scripts.Leave)

        return frame
    end

    function B.I.CreateScroll(parent, name, update, scrollname, button, buttonheight)

        local list = CreateFrame("Frame", name, parent, "BackdropTemplate")
        list:SetPoint("TOPLEFT", parent, "TOPLEFT")
        list:SetPoint("BOTTOMRIGHT", parent, "BOTTOMRIGHT")
        list:SetWidth(parent:GetWidth())

        parent.list = list
        parent.scroll = B.I.ScrollFrame(list, scrollname, update, button, buttonheight)
    end

    local function create_scroll(parent, name, update, scrollname, button, buttonheight)

        local list = CreateFrame("Frame", name, parent, "BackdropTemplate")
        list:SetPoint("TOPLEFT", parent, "TOPLEFT")
        list:SetPoint("BOTTOMRIGHT", parent, "BOTTOMRIGHT")
        list:SetWidth(parent:GetWidth())

        parent.list = list
        parent.scroll = B.I.ScrollFrame(list, scrollname, update, button, buttonheight)
    end

--[[//////////////////////////////////////////////////

    CREATE COMMON FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]

    B.I.Button = {}
    function B.I.Button.Create(index, parent, splitstack)

        local button = CreateFrame("Button", nil, parent, "SecureActionButtonTemplate")

        button:SetID(index)
        button:SetHeight(T.BUTTON_HEIGHT)
        button:SetWidth(T.BUTTON_HEIGHT)
        button:SetFrameStrata("HIGH")

        local bg = button:CreateTexture(nil, "BACKGROUND")
        bg:SetAllPoints()
        bg:SetTexture(T.MEDIABLANK)
        bg:SetVertexColor(T.TRANSPARENCY_SLOT.r, T.TRANSPARENCY_SLOT.g, T.TRANSPARENCY_SLOT.b, T.TRANSPARENCY_SLOT.a)
        button.bg = bg

        local icon = button:CreateTexture(nil, "ARTWORK", -1)
        icon:SetPoint("CENTER")
        icon:SetHeight(T.BUTTON_HEIGHT - 2)
        icon:SetWidth(T.BUTTON_HEIGHT - 2)
        icon:SetTexture("")
        icon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
        button.icon = icon

        local search = button:CreateTexture(nil, "OVERLAY")
        search:SetAllPoints()
        search:SetTexture(T.MEDIABLANK)
        search:SetVertexColor(0, 0, 0, 0.8)
        search:Hide()
        button.search = search

        local highlight = button:CreateTexture(nil, "HIGHLIGHT")
        highlight:SetColorTexture(1, 1, 1, 0.2)
        highlight:SetAllPoints()
        highlight:SetBlendMode("ADD")
        button:SetHighlightTexture(highlight)

        button.text = button:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        button.text:SetTextColor(1, 1, 1)
        button.text:SetFont(T.FONTS['quantity'], math.floor((T.BUTTON_HEIGHT * 14)/32), T.FONTS['quantityoutline'] ~= "NONE" and T.FONTS['quantityoutline'] or nil)
        button.text:SetJustifyH("LEFT")
        button.text:SetJustifyV("CENTER")
        button.text:SetWordWrap(false)
        button.text:SetPoint("RIGHT", -3)
        button.text:SetPoint("BOTTOM", 0, 3)

        button.ilvl = button:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        button.ilvl:SetFont(T.FONTS['quantity'], math.floor((T.BUTTON_HEIGHT * 14)/32), T.FONTS['quantityoutline'] ~= "NONE" and T.FONTS['quantityoutline'] or nil)
        button.ilvl:SetJustifyH("LEFT")
        button.ilvl:SetJustifyV("CENTER")
        button.ilvl:SetWordWrap(false)
        button.ilvl:SetPoint("LEFT", 1)
        button.ilvl:SetPoint("BOTTOM", 0, 3)

        button:RegisterForClicks("LeftButtonDown", "RightButtonDown")
        --button:SetAttribute("type", "macro")
        --button:SetAttribute("type2", "item")
        --button:SetAttribute("type", "click")
        --button:SetAttribute("click", "leftbutton")
        button:HookScript("PreClick", B.I.Button.PreClick)
        --button:HookScript("OnClick", B.I.Button.Click)

        if splitstack then button.SplitStack = splitstack end

        return button
    end

    function B.I.Button:Update(elapsed)
        if not elapsed then return end
        if IsShiftKeyDown() and (IsModifiedClick("COMPAREITEMS") or GetCVarBool("alwaysCompareItems")) then
            GameTooltip_ShowCompareItem(GameTooltip)
            return
        end

        if GameTooltip.shoppingTooltips then -- Reset compares
            for _, frame in pairs(GameTooltip.shoppingTooltips) do
                frame:Hide()
            end
        end

        if IsControlKeyDown() then
            ShowInspectCursor()
        elseif T.ISMERCHANTOPEN and MerchantFrame.selectedTab == 1 and T.F.mainframe.lastbank == "character" and self.sell then
            SetCursor('BUY_CURSOR')
        else
            ResetCursor()
        end
    end


    function B.I.Button:PreClick(button)

        if IsModifiedClick("CHATLINK") and T.F.mainframe.searchbar:HasFocus() and self.link then
            T.F.mainframe.searchbar:SetText(self.link)
        end
    end

    function B.I.Button:Enter(cursorUpdate)
        --print(self:GetParent():GetID().." "..self:GetID())
        if self.link then
            local parent = self:GetParent()

            if cursorUpdate == nil or cursorUpdate == true then self:SetScript("OnUpdate", B.I.Button.Update) end
            GameTooltip:SetOwner(self, "ANCHOR_NONE")
            GameTooltip:ClearAllPoints()
            GameTooltip:SetPoint("TOPRIGHT", self, "TOPLEFT", -10, -20)

            if strmatch(self.link, "|Hbattlepet:") then
                -- Battle pet link
                local _, speciesID, level, breedQuality, maxHealth, power, speed, name = strsplit(":", self.link)
                BattlePetToolTip_Show(tonumber(speciesID), tonumber(level), tonumber(breedQuality), tonumber(maxHealth), tonumber(power), tonumber(speed), name)
            elseif not T.F.mainframe.alt and parent.name == "characterbag" then
                GameTooltip:SetBagItem(parent:GetID(), self:GetID()) -- To have soulbound vs bind on picked up working
            else
                GameTooltip:SetHyperlink(self.link)
            end
            if self.quest then
                GameTooltip:SetBackdropBorderColor(1, 1, 0)
            elseif self.quality then
                local r, g, b = GetItemQualityColor(self.quality)
                GameTooltip:SetBackdropBorderColor(r, g, b)
            end
            GameTooltip:Show()
        end
    end

    function B.I.Button:Leave(...)
        ResetCursor()
        self:SetScript("OnUpdate", nil)
        GameTooltip_Hide() -- Blizzard function to close both gametooltip and battle pets
    end

    B.I.List = {}
    function B.I.List.Button(index, parent)

        local button = CreateFrame("Button", nil, parent, "BackdropTemplate")

        button:SetHeight(32)
        button:SetWidth(parent:GetWidth())
        button:SetPoint("TOP")
        button:SetPoint("LEFT")
        button:SetPoint("RIGHT")
        button:SetBackdrop({bgFile = T.MEDIABLANK, edgeFile = T.MEDIABLANK, edgeSize = 1})
        button:SetBackdropColor(0, 0, 0, 0.7)
        button:SetBackdropBorderColor(0, 0, 0, 1)

        local highlight = button:CreateTexture(nil, "HIGHLIGHT")

        highlight:SetHeight(button:GetHeight() - 2)
        highlight:SetWidth(button:GetWidth() - 2)
        highlight:SetPoint("CENTER")
        highlight:SetColorTexture(1, 1, 1, 0.1)
        highlight:SetBlendMode("ADD")
        button:SetHighlightTexture(highlight)

        local icon = button:CreateTexture(nil, "ARTWORK", -1)

        icon:SetHeight(28)
        icon:SetWidth(28)
        icon:SetTexture("")
        icon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
        icon:SetPoint("TOPLEFT", 2, -2)
        button.icon = icon

        button.text = button:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        button.text:SetTextColor(1, 1, 1)
        button.text:SetFont(T.FONTS['quantity'], 13, T.FONTS['quantityoutline'] ~= "NONE" and T.FONTS['quantityoutline'] or nil)
        button.text:SetJustifyH("LEFT")
        button.text:SetJustifyV("CENTER")
        button.text:SetWordWrap(false)
        button.text:SetPoint("RIGHT", icon)
        button.text:SetPoint("BOTTOM", icon, 0, 3)

        button.ilvl = button:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        button.ilvl:SetFont(T.FONTS['quantity'], 13, T.FONTS['quantityoutline'] ~= "NONE" and T.FONTS['quantityoutline'] or nil)
        button.ilvl:SetJustifyH("LEFT")
        button.ilvl:SetJustifyV("CENTER")
        button.ilvl:SetWordWrap(false)
        button.ilvl:SetPoint("LEFT", icon)
        button.ilvl:SetPoint("BOTTOM", icon, 0, 1)

        button.rightstring = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        button.rightstring:SetJustifyH("CENTER")
        button.rightstring:SetJustifyV("CENTER")
        button.rightstring:SetPoint("RIGHT", -10, 0)
        button.rightstring:SetWidth(45)
        button.rightstring:SetWordWrap(false)
        button.rightstring:SetTextColor(1, 1, 1)

        button.rightstring.bg = button:CreateTexture(nil, "ARTWORK")
        button.rightstring.bg:SetPoint("RIGHT", -2, 0)
        button.rightstring.bg:SetHeight(button:GetHeight() - 4)
        button.rightstring.bg:SetWidth(button.rightstring:GetWidth() + 15)
        button.rightstring.bg:SetTexture(T.MEDIABLANK)
        button.rightstring.bg:SetVertexColor(0.05, 0.05, 0.05, 1)

        button.leftstring = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        button.leftstring:SetJustifyH("LEFT")
        button.leftstring:SetJustifyV("CENTER")
        button.leftstring:SetWordWrap(false)
        button.leftstring:SetWidth(button:GetWidth() - button.rightstring.bg:GetWidth())

        button.altstring = button:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
        button.altstring:SetJustifyH("CENTER")
        button.altstring:SetJustifyV("TOP")
        button.altstring:SetWidth(button.rightstring.bg:GetWidth() - 4)
        button.altstring:SetPoint("RIGHT", -2, 5)
        button.altstring:SetWordWrap(false)
        button.altstring:SetTextColor(1, 1, 1)

        button.altstring.server = button:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
        button.altstring.server:SetJustifyH("CENTER")
        button.altstring.server:SetJustifyV("BOTTOM")
        button.altstring.server:SetWidth(button.rightstring.bg:GetWidth() - 4)
        button.altstring.server:SetPoint("RIGHT", -2, -5)
        button.altstring.server:SetFont(button.altstring.server:GetFont(), 9)
        button.altstring.server:SetWordWrap(false)
        button.altstring.server:SetTextColor(1, 1, 1)

        button.altstring.faction = button:CreateTexture(nil, "ARTWORK")
        button.altstring.faction:SetPoint("RIGHT", 2, 0)
        button.altstring.faction:SetHeight(button:GetHeight() - 4)
        button.altstring.faction:SetWidth(3)
        button.altstring.faction:SetTexture(T.MEDIABLANK)
        button.altstring.faction:SetVertexColor(0.02, 0.76, 0.31, 1)

        return button
    end

    B.I.TabList = {}
    function B.I.TabList.Button(index, parent)

        local button = CreateFrame("Button", nil, parent, "BackdropTemplate")

        button:SetHeight(28)
        button:SetWidth(parent:GetWidth())
        button:SetPoint("TOP")
        button:SetPoint("LEFT")
        button:SetPoint("RIGHT")
        button:SetBackdrop({bgFile = T.MEDIABLANK, edgeFile = T.MEDIABLANK, edgeSize = 1})
        button:SetBackdropColor(0, 0, 0, 0.7)
        button:SetBackdropBorderColor(0, 0, 0, 1)

        local highlight = button:CreateTexture(nil, "HIGHLIGHT")

        highlight:SetHeight(button:GetHeight() - 2)
        highlight:SetWidth(button:GetWidth() - 2)
        highlight:SetPoint("CENTER")
        highlight:SetColorTexture(1, 1, 1, 0.1)
        highlight:SetBlendMode("ADD")
        button:SetHighlightTexture(highlight)

        local icon = button:CreateTexture(nil, "ARTWORK", -1)

        icon:SetHeight(24)
        icon:SetWidth(24)
        icon:SetTexture("")
        icon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
        icon:SetPoint("TOPLEFT", 2, -2)
        button.icon = icon

        button.text = button:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        button.text:SetTextColor(1, 1, 1)
        button.text:SetFont(T.FONTS['quantity'], 12, T.FONTS['quantityoutline'] ~= "NONE" and T.FONTS['quantityoutline'] or nil)
        button.text:SetJustifyH("LEFT")
        button.text:SetJustifyV("CENTER")
        button.text:SetWordWrap(false)
        button.text:SetPoint("RIGHT", icon)
        button.text:SetPoint("BOTTOM", icon, 0, 3)

        button.ilvl = button:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        button.ilvl:SetFont(T.FONTS['quantity'], 12, T.FONTS['quantityoutline'] ~= "NONE" and T.FONTS['quantityoutline'] or nil)
        button.ilvl:SetJustifyH("LEFT")
        button.ilvl:SetJustifyV("CENTER")
        button.ilvl:SetWordWrap(false)
        button.ilvl:SetPoint("LEFT", icon, 1)
        button.ilvl:SetPoint("BOTTOM", icon, 0, 1)

        local price = button:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
        price:SetTextColor(1, 1, 1)
        price:SetFont(price:GetFont(), 12)
        price:SetJustifyH("CENTER")
        price:SetJustifyV("CENTER")
        price:SetWordWrap(false)
        price:SetWidth(100)
        price:SetPoint("RIGHT", -10, 0)
        button.rightstring = price

        button.rightstring.bg = button:CreateTexture(nil, "ARTWORK")
        button.rightstring.bg:SetPoint("RIGHT", -2, 0)
        button.rightstring.bg:SetHeight(button:GetHeight() - 4)
        button.rightstring.bg:SetWidth(button.rightstring:GetWidth() + 15)
        button.rightstring.bg:SetTexture(T.MEDIABLANK)
        button.rightstring.bg:SetVertexColor(0.05, 0.05, 0.05, 1)

        button.rightstring.time = button:CreateTexture(nil, "ARTWORK")
        button.rightstring.time:SetPoint("RIGHT", -3, 0)
        button.rightstring.time:SetHeight(button:GetHeight())
        button.rightstring.time:SetWidth(3)
        button.rightstring.time:SetTexture(T.MEDIABLANK)
        button.rightstring.time:SetVertexColor(0.02, 0.76, 0.31, 1)

        button.leftstring = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        button.leftstring:SetJustifyH("LEFT")
        button.leftstring:SetJustifyV("CENTER")
        button.leftstring:SetWordWrap(false)
        button.leftstring:SetFont(price:GetFont(), 10)
        button.leftstring:SetWidth(button:GetWidth() - button.rightstring.bg:GetWidth())

        return button
    end

    B.I.Tab = {}
    function B.I.Tab.Create(parent, name, inherit)
        local button = CreateFrame("Button", name, parent, inherit)
        --B.print(button)
        button:SetHeight(T.BUTTON_HEIGHT)
        button:SetWidth(T.BUTTON_HEIGHT)
        button:SetFrameStrata("HIGH")

        button:SetBackdrop({bgFile = T.MEDIABLANK, edgeFile = T.MEDIABLANK, edgeSize = 1})
        button:SetBackdropColor(0, 0, 0, 0.2)
        button:SetBackdropBorderColor(0, 0, 0, 1)

        local normal = button:CreateTexture(nil, "OVERLAY")
        normal:SetPoint("CENTER")
        normal:SetHeight(button:GetHeight() - 2)
        normal:SetWidth(button:GetWidth() - 2)
        normal:SetTexture("")
        normal:SetTexCoord(0.1, 0.9, 0.1, 0.9)
        button:SetNormalTexture(normal)

        local highlight = button:CreateTexture(nil, "HIGHLIGHT")
        highlight:SetColorTexture(1, 1, 1, 0.2)
        highlight:SetAllPoints()
        highlight:SetBlendMode("ADD")
        button:SetHighlightTexture(highlight)

        local search = button:CreateTexture(nil, "OVERLAY")

        search:SetAllPoints(normal)
        search:SetTexture(T.MEDIABLANK)
        search:SetVertexColor(0, 0, 0, 0.8)
        search:Hide()
        button.search = search

        return button
    end

    B.I.Frame = {}
    function B.I.Frame.Create(parent, pframe, index)
        -- Setup base for holding our buttons
        pframe.buttons = {}

        local frame = CreateFrame("Frame", nil, parent, "BackdropTemplate")

        frame:SetPoint("BOTTOM")
        frame:SetPoint("TOPLEFT", 10, -20)
        frame:SetFrameStrata("HIGH")

        --if index then frame:SetID(index) end
        if index and index > 1 then frame:Hide() end

        pframe.frame = frame
    end

    B.I.Grid = {}
    function B.I.Grid.Update(parent)
        -- Dynamically create a grid based on the parent.buttons array of buttons
        local x, buttons, button, lastbutton, lastrow, rows, num
        buttons = parent.buttons
        rows = 0
        num = 0

        for x = 1, #buttons do
            button = buttons[x]
            button:ClearAllPoints()

            if x == 1 then -- First slot
                button:SetPoint("TOPLEFT")
                rows, num = 1, 1
                lastrow = button

            elseif num == T.ROW then -- Last column of the row
                button:SetPoint("TOP", lastrow, "BOTTOM", 0, -T.FRAMEWHITESPACE)
                rows, num = rows + 1, 1
                lastrow = button
            else
                button:SetPoint("TOPLEFT", lastbutton, "TOPRIGHT", T.FRAMEWHITESPACE, 0)
                num = num + 1
            end

            lastbutton = button
        end

        return rows
    end

    B.I.PrevButton = {}
    function B.I.PrevButton.Create(frame)

        local button = CreateFrame("Button", nil, frame)
        
        button:SetWidth(24)
        button:SetHeight(24)
        button:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Up")
        button:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Down")
        button:SetDisabledTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Disabled")
        button:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight")
        button:Disable()

        if T.SKIN then F.ReskinArrow(button, "left") end -- Aurora

        return button
    end

    B.I.NextButton = {}
    function B.I.NextButton.Create(frame)

        local button = CreateFrame("Button", nil, frame)
        
        button:SetWidth(24)
        button:SetHeight(24)
        button:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Up")
        button:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Down")
        button:SetDisabledTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Disabled")
        button:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight")
        button:Disable()

        if T.SKIN then F.ReskinArrow(button, "right") end -- Aurora

        return button
    end

    function B.I.Backdrop(frame)

        frame:SetBackdrop({ bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
                            edgeFile = "Interface\\ChatFrame\\ChatFrameBackground",
                            tile = false, tileSize = 32, edgeSize = 1,
                            insets = { left = 0, right = 0, top = 0, bottom = 0 }
        })
        frame:SetBackdropBorderColor(0, 0, 0, 1)
        frame:SetBackdropColor(0, 0, 0, 0.5)
        frame:EnableMouse(true)
    end

    B.I.GradientButton = {}
    function B.I.GradientButton.Create(button, orientation)

        button:SetNormalTexture("")
        button:SetHighlightTexture("")
        button:SetPushedTexture("")
        button:SetDisabledTexture("")

        B.I.RemoveEdge(button)
        B.I.Create:Backdrop(button)

        B.I.Create:Gradient(button, orientation)
    end

    B.I.Reskin = {}
    function B.I.Reskin:CheckBox(checkbox)

        checkbox:SetNormalTexture("")
        checkbox:SetPushedTexture("")

        local ch = checkbox:GetCheckedTexture()
        ch:SetTexture("Interface\\Buttons\\UI-CheckBox-Check")
        ch:SetTexCoord(0, 1, 0, 1)

        local bg = B.I.Create:Backdrop(checkbox)
        bg:SetPoint("TOPLEFT", 4, -4)
        bg:SetPoint("BOTTOMRIGHT", -4, 4)
    end

    function B.I.Reskin:Button(button)

        button:SetNormalTexture("")
        button:SetHighlightTexture("")
        button:SetPushedTexture("")
        button:SetDisabledTexture("")

        B.I.RemoveEdge(button)
        B.I.Create:Gradient(button, "VERTICAL")
        button.bg = B.I.Create:Backdrop(button)

        button:HookScript("OnEnter", B.I.Reskin.Button_OnEnter)
        button:HookScript("OnLeave", B.I.Reskin.Button_OnLeave)
    end

    function B.I.Reskin:Button_OnEnter()
        self.bg:SetBackdropColor(1, 1, 1, 0.075)
        self.bg:SetBackdropBorderColor(1, 1, 1, 0.1)
    end

    function B.I.Reskin:Button_OnLeave()
        self.bg:SetBackdropColor(0, 0, 0, 0.2)
        self.bg:SetBackdropBorderColor(0, 0, 0, 0.8)
    end

    function B.I.Reskin:Input(editbox)

        editbox:SetTextColor(1, 1, 1, 1)
        editbox:SetTextInsets(5, 0, 0, 0)

        B.I.RemoveEdge(editbox)
        
        editbox:SetBackdrop({   bgFile = T.MEDIABLANK,
                                edgeFile = T.MEDIABLANK,
                                edgeSize = 1,
                                insets = { left = 0, right = 0, top = 0, bottom = 0 }
        })
        editbox:SetBackdropColor(0, 0, 0, 0.6)
        editbox:SetBackdropBorderColor(0, 0, 0, 0.8)
        editbox:SetFontObject("GameFontNormal")

        editbox:HookScript("OnEscapePressed", function(self) self:ClearFocus() end)
    end


    B.I.Create = {}
    function B.I.Create:Gradient(parent, orientation)

        local gradient = parent:CreateTexture(nil, "BACKGROUND")

        gradient:SetColorTexture(1, 1, 1)
        gradient:SetAllPoints()
        gradient:SetGradientAlpha(orientation, 0, 0, 0, 0, 0.3, 0.3, 0.3)

        return gradient
    end

    function B.I.Create:Backdrop(parent)
        local frame = CreateFrame("Frame", nil, parent, "BackdropTemplate")

        frame:SetBackdrop({bgFile = T.MEDIABLANK,
                            edgeFile = T.MEDIABLANK,
                            edgeSize = 1,
                            insets = { left = 0, right = 0, top = 0, bottom = 0 }
        })
        frame:SetBackdropColor(0, 0, 0, 0.3)
        frame:SetBackdropBorderColor(0, 0, 0, 0.8)
        frame:SetAllPoints()

        return frame
    end


    function B.I.RemoveEdge(frame)

        local name = frame:GetName()
        local left = frame.Left or name and _G[name.."Left"] or nil
        local middle = frame.Middle or name and (_G[name.."Middle"] or _G[name.."Mid"]) or nil
        local right = frame.Right or name and _G[name.."Right"] or nil

        if left then left:Hide() end
        if middle then middle:Hide() end
        if right then right:Hide() end
    end

    function B.I.Input(editbox)

        editbox:SetTextColor(1, 1, 1)
        editbox:SetTextInsets(5, 0, 0, 0)

        B.I.RemoveEdge(editbox)
        
        editbox:SetBackdrop({   bgFile = T.MEDIABLANK,
                                edgeFile = T.MEDIABLANK,
                                edgeSize = 1,
                                insets = { left = 0, right = 0, top = 0, bottom = 0 }
        })
        editbox:SetBackdropColor(0, 0, 0, 0.8)
        editbox:SetBackdropBorderColor(1, 1, 1, 0.1)
        editbox:SetFontObject("GameFontNormal")

        editbox:HookScript("OnEscapePressed", function(self) self:ClearFocus() end)
    end

    function B.I.ScrollFrame(parent, name, onUpdate, buttons, height)

        local scrollframe = B.I.Scroll.Create(parent, name, onUpdate, height)
        local i, currentrow, previousrow

        parent.buttons = {}

        for i = 1, B.RowsToFit(parent:GetParent(), height) do
            parent.buttons[i] = buttons(i, parent)
            currentrow = parent.buttons[i]
            if i == 1 then
                currentrow:SetPoint("TOP")
            else
                currentrow:SetPoint("TOP", previousrow, "BOTTOM", 0, -1)
            end
            currentrow:Show()
            previousrow = currentrow
        end

        return scrollframe
    end

    B.I.Scroll = {}
    function B.I.Scroll.Create(parent, name, onUpdate, height)

        --[[note the anchors: the area of the scrollframe is the scrollable area
            that intercepts mousewheel to scroll. it does not include the scrollbar,
            which is anchored off the right ]]

        local scroll = CreateFrame("ScrollFrame", name, parent, "FauxScrollFrameTemplate")
        scroll:SetPoint("TOPLEFT", parent:GetParent(), "TOPLEFT")
        scroll:SetPoint("BOTTOMRIGHT", parent:GetParent(), "BOTTOMRIGHT")
        scroll.onUpdate = onUpdate
        scroll.height = height

        scroll:SetScript("OnShow", onUpdate)
        scroll:SetScript("OnVerticalScroll", B.I.Scroll.VerticalScroll)
        scroll.ScrollBar:SetMinMaxValues(0, 100)
        scroll.ScrollBar:Hide()

        return scroll
    end

    function B.I.Scroll:VerticalScroll(offset)
        FauxScrollFrame_OnVerticalScroll(self, offset, self.height or 32, self.onUpdate)
    end

    B.I.ColorPicker = {}
    function B.I.ColorPicker.Create(r, g, b, a, changedCallback)
        ColorPickerFrame.func, ColorPickerFrame.opacityFunc, ColorPickerFrame.cancelFunc = nil, nil, nil
        ColorPickerFrame:SetColorRGB(r, g, b);
        ColorPickerFrame.hasOpacity, ColorPickerFrame.opacity = (a ~= nil), a;
        ColorPickerFrame.previousValues = {r, g, b, a};
        ColorPickerFrame.func, ColorPickerFrame.opacityFunc, ColorPickerFrame.cancelFunc = changedCallback, changedCallback, changedCallback;
        ColorPickerFrame:Hide(); -- Need to run the OnShow handler.
        ColorPickerFrame:Show();
    end

--[[//////////////////////////////////////////////////

    ANCHOR FUNCTIONS

----\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\]]
    
    function B.I.AnchorBagsTopLeft()
        -- DEFAULT LAYOUT
        T.F.mainframe.layout = 1
        B.AnchorFrames("TOP", "BOTTOM", "LEFT", "RIGHT")
    end

    function B.I.AnchorBagsBottomLeft()

        T.F.mainframe.layout = 2
        B.AnchorFrames("BOTTOM", "TOP", "LEFT", "RIGHT")
    end

    function B.I.AnchorBagsTopRight()

        T.F.mainframe.layout = 3
        B.AnchorFrames("TOP", "BOTTOM", "RIGHT", "LEFT")
    end

    function B.I.AnchorBagsBottomRight()

        T.F.mainframe.layout = 4
        B.AnchorFrames("BOTTOM", "TOP", "RIGHT", "LEFT")
    end

    function B.I.Main.Frame.ButtonHighlight(button, orientation)
        local highlight = button:GetHighlightTexture()

        highlight:ClearAllPoints()
        highlight:SetPoint(orientation.."LEFT", button, orientation.."LEFT", 0, orientation == "TOP" and 1 or -1)
        highlight:SetPoint(orientation.."RIGHT", button, orientation.."RIGHT", 0, orientation == "TOP" and 1 or -1)
        highlight:SetHeight(2)
    end

    function B.I.Main.Frame.ButtonOrder(orientation)

        T.F.mainframe.optionbutton:ClearAllPoints()
        T.F.mainframe.bankbutton:ClearAllPoints()
        T.F.mainframe.reagentbutton:ClearAllPoints()
        T.F.mainframe.characterbutton:ClearAllPoints()
        T.F.mainframe.voidbutton:ClearAllPoints()
        T.F.mainframe.guildbutton:ClearAllPoints()
        T.F.mainframe.searchbar:ClearAllPoints()
        T.F.mainframe.countbutton:ClearAllPoints()

        if orientation == "LEFT" then
            B.I.Main.Frame.ButtonLeft("LEFT", "RIGHT")
        else
            B.I.Main.Frame.ButtonRight("RIGHT", "LEFT")
        end
    end

    function B.I.Main.Frame.ButtonLeft(startpoint, direction)

        T.F.mainframe.optionbutton:SetPoint(startpoint, T.F.mainframe, startpoint)
        T.F.mainframe.optionbutton:SetPoint("BOTTOM", T.F.mainframe, "BOTTOM")
        T.F.mainframe.bankbutton:SetPoint(startpoint, T.F.mainframe.optionbutton, direction)
        T.F.mainframe.reagentbutton:SetPoint(startpoint, T.F.mainframe.bankbutton, direction)
        T.F.mainframe.characterbutton:SetPoint(startpoint, T.F.mainframe.reagentbutton, direction)
        T.F.mainframe.voidbutton:SetPoint(startpoint, T.F.mainframe.characterbutton, direction)
        T.F.mainframe.guildbutton:SetPoint(startpoint, T.F.mainframe.voidbutton, direction)
        T.F.mainframe.searchbar:SetPoint(startpoint, T.F.mainframe.guildbutton, direction, 10, 0)
        T.F.mainframe.countbutton:SetPoint(startpoint, T.F.mainframe.searchbar, direction)
    end

    function B.I.Main.Frame.ButtonRight(startpoint, direction)

        T.F.mainframe.optionbutton:SetPoint(startpoint, T.F.mainframe, startpoint)
        T.F.mainframe.optionbutton:SetPoint("BOTTOM", T.F.mainframe, "BOTTOM")
        T.F.mainframe.guildbutton:SetPoint(startpoint, T.F.mainframe.optionbutton, direction)
        T.F.mainframe.voidbutton:SetPoint(startpoint, T.F.mainframe.guildbutton, direction)
        T.F.mainframe.characterbutton:SetPoint(startpoint, T.F.mainframe.voidbutton, direction)
        T.F.mainframe.reagentbutton:SetPoint(startpoint, T.F.mainframe.characterbutton, direction)
        T.F.mainframe.bankbutton:SetPoint(startpoint, T.F.mainframe.reagentbutton, direction)
        T.F.mainframe.searchbar:SetPoint(startpoint, T.F.mainframe.bankbutton, direction, -10, 0)
        T.F.mainframe.countbutton:SetPoint(startpoint, T.F.mainframe.searchbar, direction)
    end

    function B.AnchorFrames(startpoint, endpoint, from, to)

        B.I.Main.Frame.ButtonOrder(from)

        _G['BWdropDownMenu']:ClearAllPoints()
        _G['BWdropDownMenu'].point = startpoint..from
        _G['BWdropDownMenu'].relativePoint = endpoint..from

        -- bank
        B.Bank:Anchor(startpoint, endpoint, from, to)
        -- reagent
        B.Reagent:Anchor(startpoint, endpoint, from, to)
        -- character
        B.Character:Anchor(startpoint, endpoint, from, to)
        B.Equipment:Anchor(startpoint, endpoint, from, to)
        --B.Farm:Anchor(startpoint, endpoint, from, to)
        -- void
        B.Void:Anchor(startpoint, endpoint, from, to)
        -- guild
        B.Guild:Anchor(startpoint, endpoint, from, to)
        -- search
        B.Search:Anchor(startpoint, endpoint, from, to)
        -- auctions
        B.Auctions:Anchor(startpoint, endpoint, from, to)
        -- mail
        B.Mail:Anchor(startpoint, endpoint, from, to)
    end

    function B.AnchorBags(parent)

        T.F.mainframe.character:ClearAllPoints()
        if T.F.mainframe.layout < 3 then -- left side
            T.F.mainframe.character:SetPoint("TOPLEFT", parent, "TOPRIGHT", 36, 0)
        else -- right side
            T.F.mainframe.character:SetPoint("TOPRIGHT", parent, "TOPLEFT", -36, 0)
        end
    end

    local function resetbg_button(button)

        if not button.name or button.quality < 2 then
            button.bg:SetVertexColor(T.TRANSPARENCY_SLOT.r, T.TRANSPARENCY_SLOT.g, T.TRANSPARENCY_SLOT.b, T.TRANSPARENCY_SLOT.a)
        end
    end

    function B.AdjustTransparency()
        local buttons

        T.F.mainframe.bank:SetBackdropColor(T.TRANSPARENCY.r, T.TRANSPARENCY.g, T.TRANSPARENCY.b, T.TRANSPARENCY.a)
        T.F.mainframe.bank:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
        T.F.mainframe.reagent:SetBackdropColor(T.TRANSPARENCY.r, T.TRANSPARENCY.g, T.TRANSPARENCY.b, T.TRANSPARENCY.a)
        T.F.mainframe.reagent:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
        T.F.mainframe.character:SetBackdropColor(T.TRANSPARENCY.r, T.TRANSPARENCY.g, T.TRANSPARENCY.b, T.TRANSPARENCY.a)
        T.F.mainframe.character:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
        T.F.mainframe.character.equipment:SetBackdropColor(T.TRANSPARENCY.r, T.TRANSPARENCY.g, T.TRANSPARENCY.b, T.TRANSPARENCY.a)
        T.F.mainframe.character.equipment:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
        --T.F.mainframe.farm:SetBackdropColor(T.TRANSPARENCY.r, T.TRANSPARENCY.g, T.TRANSPARENCY.b, T.TRANSPARENCY.a)
        --T.F.mainframe.farm:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
        T.F.mainframe.void:SetBackdropColor(T.TRANSPARENCY.r, T.TRANSPARENCY.g, T.TRANSPARENCY.b, T.TRANSPARENCY.a)
        T.F.mainframe.void:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
        T.F.mainframe.guild:SetBackdropColor(T.TRANSPARENCY.r, T.TRANSPARENCY.g, T.TRANSPARENCY.b, T.TRANSPARENCY.a)
        T.F.mainframe.guild:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)

        T.F.mainframe.bank.tooltiptoggle:SetBackdropColor(T.THEME.r, T.THEME.g, T.THEME.b, T.THEME.a)
        T.F.mainframe.bank.tooltiptoggle:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
        T.F.mainframe.bank.bagstoggle:SetBackdropColor(T.THEME.r, T.THEME.g, T.THEME.b, T.THEME.a)
        T.F.mainframe.bank.bagstoggle:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
        T.F.mainframe.character.tooltiptoggle:SetBackdropColor(T.THEME.r, T.THEME.g, T.THEME.b, T.THEME.a)
        T.F.mainframe.character.tooltiptoggle:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
        T.F.mainframe.character.bagstoggle:SetBackdropColor(T.THEME.r, T.THEME.g, T.THEME.b, T.THEME.a)
        T.F.mainframe.character.bagstoggle:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)
        T.F.mainframe:SetBackdropColor(T.THEME.r, T.THEME.g, T.THEME.b, T.THEME.a)
        T.F.mainframe:SetBackdropBorderColor(T.THEME.r, T.THEME.g, T.THEME.b, T.THEME.a)
        T.F.mainframe.countbutton:SetBackdropColor(T.THEME.r, T.THEME.g, T.THEME.b, 1)

        for x = 1, #T.F.mainframe.bank.buttons do
            resetbg_button(T.F.mainframe.bank.buttons[x])
        end

        for b = 1, T.REAGENT_PAGES do
            buttons = T.F.mainframe.reagent[b].buttons
            for x = 1, #buttons do
                resetbg_button(buttons[x])
            end
        end

        for x = 1, #T.F.mainframe.character.buttons do
            resetbg_button(T.F.mainframe.character.buttons[x])
        end

        for x = 1, #T.F.mainframe.character.equipment.buttons do
            resetbg_button(T.F.mainframe.character.equipment.buttons[x])
        end

        for b = 1, T.VOID_PAGES do
            buttons = T.F.mainframe.void[b].buttons
            for x = 1, #buttons do
                resetbg_button(buttons[x])
            end
        end

        for b = 1, T.MAX_GUILDBANK_TABS do
            buttons = T.F.mainframe.guild[b].buttons
            for x = 1, #buttons do
                resetbg_button(buttons[x])
            end
        end
    end

    local function resetfont(button)
        local _, size = button:GetFont()
        button:SetFont(T.FONTS['quantity'], size, T.FONTS['quantityoutline'] ~= "NONE" and T.FONTS['quantityoutline'] or nil)
    end

    local function resetfont_button(button)

        resetfont(button.text)
        resetfont(button.ilvl)
    end

    function B.ResetFontButtons()
        local b, x, buttons

        for x = 1, #T.F.mainframe.bank.buttons do
            resetfont_button(T.F.mainframe.bank.buttons[x])
        end

        for b = 1, T.REAGENT_PAGES do
            buttons = T.F.mainframe.reagent[b].buttons
            for x = 1, #buttons do
                resetfont_button(buttons[x])
            end
        end

        for x = 1, #T.F.mainframe.character.buttons do
            resetfont_button(T.F.mainframe.character.buttons[x])
        end

        for x = 1, #T.F.mainframe.character.equipment.buttons do
            resetfont_button(T.F.mainframe.character.equipment.buttons[x])
        end

        for b = 1, T.VOID_PAGES do
            buttons = T.F.mainframe.void[b].buttons
            for x = 1, #buttons do
                resetfont_button(buttons[x])
            end
        end

        for b = 1, T.MAX_GUILDBANK_TABS do
            buttons = T.F.mainframe.guild[b].buttons
            for x = 1, #buttons do
                resetfont_button(buttons[x])
            end
        end

        for x = 1, #T.F.mainframe.search.list.buttons do
            resetfont_button(T.F.mainframe.search.list.buttons[x])
        end

        for x = 1, #T.F.mainframe.character.auctions.list.buttons do
            resetfont_button(T.F.mainframe.character.auctions.list.buttons[x])
        end

        for x = 1, #T.F.mainframe.character.mail.list.buttons do
            resetfont_button(T.F.mainframe.character.mail.list.buttons[x])
        end

        local _, size = T.F.mainframe.countbutton.labelcount:GetFont()
        T.F.mainframe.countbutton.labelcount:SetFont(T.CONFIG['FontQuantity'] == "default" and T.FONTS[2] or not T.CONFIG['FontQuantity'] and T.FONTS[2] or T.FONTS['quantity'], size)

        -- Reset the option menu font taint by opening and closing it during loading... 
        -- I know it's hack-ish...
        LibDD:ToggleDropDownMenu(1, nil, _G['BWdropDownMenu'], "BWoptionbutton", 0, 0, nil, nil, 2)
        LibDD:CloseDropDownMenus()
    end

